import os
from ost import seq, mol, io
import tempfile
def SaveToTempDir(objects, seq_format='fasta', structure_format='pdb'):
  """
  Take all objects and saves them to a temporary directory. The paths of the 
  saved files will be returned as a tuple. This works for alignments, and 
  structure files. The output format for sequence files and structure files can
  be changed by setting to seq_format and structure_format parameters 
  appropriately.
  """
  # create temporary directory
  tmp_dir_name=tempfile.mkdtemp()
  file_names=[]
  for index, obj in enumerate(objects):
    if isinstance(obj, seq.AlignmentHandle):
      name=os.path.join(tmp_dir_name, 'aln%02d.fasta' % (index+1))
      io.SaveAlignment(obj, name, seq_format)
      file_names.append(name)
      continue
    if isinstance(obj, seq.SequenceHandle):
      name=os.path.join(tmp_dir_name, 'seq%02d.fasta' % (index+1))
      io.SaveSequence(obj, name, seq_format)
      file_names.append(name)
      continue
    if isinstance(obj, seq.ConstSequenceList) or isinstance(obj, seq.SequenceList):
      name=os.path.join(tmp_dir_name, 'sql%02d.fasta' % (index+1))
      io.SaveSequenceList(obj, name, seq_format)
      file_names.append(name)
      continue
    if isinstance(obj, mol.EntityView) or isinstance(obj, mol.EntityHandle):
      name=os.path.join(tmp_dir_name, 'mol%02d.pdb' % (index+1))
      io.SaveEntity(obj, name, structure_format)
      file_names.append(name)
      continue
  return file_names
  
class TempDirWithFiles:
  def __init__(self, objects, seq_format='fasta', structure_format='pdb'):
    self.files=SaveToTempDir(objects, seq_format=seq_format, 
                             structure_format=structure_format)
    self.dirname=os.path.dirname(self.files[0])
  def Cleanup(self):
    import shutil
    shutil.rmtree(self.dirname)





'''
Naccess module

Author: Florian Kiefer, Gerardo Tauriello (cleanup/speedup)

This module is for calculating surface areas
from OpenStructure using the external program naccess

How To Use This Module:
 1. Import it (e.g. as "from ost.bindings import naccess")
 2. Use it (e.g. as "sasa = naccess.CalculateSurfaceArea(entity)")

Requirement:
 - naccess installed
'''

import tempfile
import subprocess
import os
import re
from ost import io, mol, settings, LogWarning

def _GetExecutable(naccess_exe):
  """
  Method to check if naccess executable is present

  :param naccess:   Explicit path to naccess executable
  :returns:         Path to the executable
  :exception:       FileNotFound if executable is not found
  """
  return settings.Locate('naccess', explicit_file_name=naccess_exe)

def _CheckNaccessRoot(naccess_root):
  """
  :return: True, if given directory contains "accall" binary and files
           "vdw.radii" and "standard.data".
  :param naccess_root: Path to naccess folder to check.
  """
  accall_exe = os.path.join(naccess_root, "accall")
  check = (os.path.exists(accall_exe) and os.access(accall_exe, os.X_OK) \
           and os.path.exists(os.path.join(naccess_root, "vdw.radii")) \
           and os.path.exists(os.path.join(naccess_root, "standard.data")))
  if not check:
    LogWarning("NACCESS: Could not find required files to launch accall " \
               "directly in %s." % naccess_root)
  return check

def _SetupFiles(entity, selection, scratch_dir, max_number_of_atoms):
  """
  Setup files for naccess calculation in temporary directory

  :param entity:              OST entity to calculate surface
  :param selection:           Calculate surface for subset of entity
  :param scratch_dir:         Scratch directory. A subfolder for temporary files
                              is created in there. If not specified, a default
                              directory is used (see :func:`tempfile.mkdtemp`).
  :param max_number_of_atoms: Max Number of atoms in the entity (i.e. is limited
                              in the default NACCESS version to 50 000)
  :returns: Tuple containing temporary directory, input filename for naccess and
            directory of the input file
  :exception: RuntimeError if selection is not valid or too many atoms
  """

  # create temporary directory
  tmp_dir_name = ""
  if scratch_dir != None:
    tmp_dir_name = tempfile.mkdtemp(dir=scratch_dir)
  else:
    tmp_dir_name = tempfile.mkdtemp()

  # select as specified by user
  if selection != "":
    entity_view = entity.Select(selection)
  else:
    entity_view = entity
  if len(entity_view.atoms) > max_number_of_atoms:
    raise RuntimeError("Too much atoms for NACCESS (> %s)" % max_number_of_atoms)
  if not entity_view.IsValid():
    raise RuntimeError("Could not create view for selection (%s)"%(selection))
  
  # write entity to tmp file
  tmp_file_name = "entity.pdb"
  tmp_file_base = os.path.join(tmp_dir_name, "entity")
  io.SavePDB(entity_view, os.path.join(tmp_dir_name, tmp_file_name))
  return (tmp_dir_name, tmp_file_name, tmp_file_base)


def _ParseAsaFile(entity, file, asa_atom):
  """
  Reads Area file (.asa) and attach asa per atom to an entitiy

  :param entity:   EntityHandle or EntityView for attaching sasa on atom level
  :param file:     Filename of area file
  :param asa_atom: Name of the float property for SASA
  """
  
  asa_fh = open(file)
  asa_lines = asa_fh.readlines()
  asa_fh.close()
  
  for l in  asa_lines:
    if l.startswith("ATOM"):
      # get res_number, chain_id and atom name
      atom_name = l[12:16]
      chain_id = l[21]
      res_number = l[22:27]
      asa = l[54:63]
      atom_name = atom_name.strip()
      chain_id = chain_id
      res_number = res_number.strip()
      asa = asa.strip()
      m = re.match(r'(?P<num>-?\d+)(?P<ins>\w)?', res_number)
      di = m.groupdict()
      
      if di["ins"] == None:
        resNum = mol.ResNum(int(di["num"]))
      else:
        resNum = mol.ResNum(int(di["num"]), di["ins"])

      a = entity.FindAtom(chain_id, resNum, atom_name)
      if(a.IsValid()):
        a.SetFloatProp(asa_atom, float(asa))
      else:
        LogWarning("NACCESS: invalid asa entry %s %s %s" \
                   % (chain_id, resNum, atom_name))
      
def _ParseRsaFile(entity, file, asa_abs, asa_rel):
  """
  Reads Area file (.rsa) and attach asa (absolute + relative) per residue to an entitiy

  :param entity:     EntityHandle or EntityView for attaching sasa on atom level
  :param file:       Filename of .rsa file
  :param asa_atom:   Name of the float property for absolute SASA
  :param asa_atom:   Name of the float property for relative SASA
  :exception:        RuntimeError if residue names are not the same
  """
  area_fh = open(file)
  area_lines = area_fh.readlines()
  area_fh.close()
  # shift first line
  area_lines = area_lines[4:]
  # parse lines
  for l in area_lines:
    if l.startswith("RES"):
      # extract data
      p = re.compile(r'\s+')
      res_name = l[3:8]
      res_name = res_name.strip()
      chain_id = l[8:9]
      res_number = l[9:14]
      res_number = res_number.strip()
      abs_all, rel_all = l[15:28].strip().split()
      m = re.match(r'(?P<num>-?\d+)(?P<ins>\w)?', res_number)
      di = m.groupdict()
      if di["ins"] == None:
        resNum = mol.ResNum(int(di["num"]))
      else:
        resNum = mol.ResNum(int(di["num"]), di["ins"])
      # set res. props
      res = entity.FindResidue(chain_id, resNum)
      if res_name == res.name:
        res.SetFloatProp(asa_rel, float(rel_all) )
        res.SetFloatProp(asa_abs, float(abs_all) )
      else:
        raise RuntimeError("Residue Names are not the same for ResNumb: %s (%s vs %s)" % (res_number, res.name, res_name))
      

def __CleanupFiles(dir_name):
  """
  Method which recursively deletes a directory

  :warning: This method removes also non-empty directories without asking, so
            be careful!
  """
  import shutil
  shutil.rmtree(dir_name)


def _RunACCALL(command, temp_dir, query):
  """
  Fast method to run the Naccess surface calculation.

  This method starts the accall binary directly and pipes in the input provided
  in *query*. This is faster than calling the "naccess" script since the script
  has a constant overhead of roughly 1.3s in each call.

  :param command:  Command to execute
  :param temp_dir: Command is executed with this working directory
  :param query:    User input to pipe into *command*
  :returns:        stdout of command
  :exception:      CalledProcessError for non-zero return value
  """

  proc = subprocess.Popen(command, cwd=temp_dir, stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE, stdin=subprocess.PIPE)
  stdout_value, stderr_value = proc.communicate(query.encode())

  # check for successful completion of naccess
  if proc.returncode != 0:
    LogWarning("WARNING: naccess error\n%s\n%s" % (stdout_value.decode(), 
                                                   stderr_value.decode()))
    raise subprocess.CalledProcessError(proc.returncode, command)

  return stdout_value.decode()


def _RunNACCESS(command, temp_dir):
  """
  Method to run the Naccess surface calculation.

  This method starts the external Naccess executable and returns the stdout.

  :param command:  Command to execute
  :param temp_dir: Command is executed with this working directory
  :returns:        stdout of command
  :exception:      CalledProcessError for non-zero return value
  """
  proc = subprocess.Popen(command, cwd=temp_dir, shell=True, 
                          stdout=subprocess.PIPE)
  stdout_value, stderr_value = proc.communicate()

  # check for successful completion of naccess
  if proc.returncode != 0:
    LogWarning("WARNING: naccess error\n%s" % stdout_value.decode())
    raise subprocess.CalledProcessError(proc.returncode, command)

  return stdout_value.decode()


def CalculateSurfaceArea(entity,  radius=1.4,  
                         include_hydrogens=False, include_hetatm = False, 
                         include_water=False, selection="", naccess_exe=None,
                         naccess_root=None, keep_files=False, asa_abs= "asaAbs",
                         asa_rel="asaRel", asa_atom="asaAtom", scratch_dir=None,
                         max_number_of_atoms=50000):
  """
  Calculates analytical the solvent accessible surface area by using the
  external naccess program

  This method calculates the molecular surface areas by invoking the external
  program naccess. First, it is checked if the naccess executable is present, then,
  the necessary files are prepared in a temporary directory and naccess is
  executed. The last step is to remove the temporary directory.


  :param entity:              OST entity to calculate surface
  :param radius:              Surface probe radius
  :param include_hydrogens:   Calculate surface including hydrogens
  :param include_hetatm:      Calculate surface including hetatms
  :param include_water:       Calculate surface including water
  :param selection:           Calculate surface for subset of entity
  :param naccess_exe:         naccess executable (full path to executable)
  :param naccess_root:        Path to folder containing "accall" binary and
                              files "vdw.radii" and "standard.data". This is the
                              fastest way to call naccess!
  :param keep_files:          If True, do not delete temporary files
  :param asa_abs:             Attaches per residue absolute SASA to specified
                              FloatProp on residue level
  :param asa_rel:             Attaches per residue relative SASA to specified
                              FloatProp on residue level
  :param asa_atom:            Attaches per atom SASA to specified FloatProp at
                              atom level
  :param scratch_dir:         Scratch directory. A subfolder for temporary files
                              is created in there. If not specified, a default
                              directory is used (see :func:`tempfile.mkdtemp`).
  :param max_number_of_atoms: Max Number of atoms in the entity (i.e. is limited
                              in the default NACCESS version to 50 000)

  :returns:                   absolute SASA calculated using asa_atom
  """
  
  # check if naccess executable is specified
  if naccess_root and _CheckNaccessRoot(naccess_root):
    # use faster, direct call to accall binary
    fast_mode = True
  else:
    # get naccess executable
    naccess_executable = _GetExecutable(naccess_exe)
    # see if we can extract naccess_root from there (fallback to old mode)
    naccess_root = os.path.dirname(naccess_executable)
    fast_mode = _CheckNaccessRoot(naccess_root)
  
  # setup files for naccess
  (naccess_data_dir, naccess_data_file, naccess_data_base) \
    = _SetupFiles(entity, selection, scratch_dir, max_number_of_atoms)

  try:
    # call naccess
    if fast_mode:
      # cook up stdin query (same logic as naccess script)
      query = "PDBFILE %s\n" \
              "VDWFILE %s\n" \
              "STDFILE %s\n" \
              "PROBE %f\n" \
              "ZSLICE 0.05\n" \
              % (naccess_data_file, os.path.join(naccess_root, "vdw.radii"),
                 os.path.join(naccess_root, "standard.data"), radius)
      if include_hydrogens:
        query += "HYDROGENS\n"
      if include_water:
        query += "WATERS\n"
      if include_hetatm:
        query += "HETATOMS\n"
      # call it
      command = os.path.join(naccess_root, "accall")
      _RunACCALL(command, naccess_data_dir, query)
    else:
      LogWarning("NACCESS: Falling back to slower call to %s." \
                 % naccess_executable)
      # set command line
      command = "%s %s -p %f " % \
                (naccess_executable, naccess_data_file, radius)
      if include_hydrogens:
        command = "%s -y" % command
      if include_water:
        command = "%s -w" % command
      if include_hetatm:
        command = "%s -h" % command
      # execute naccess
      _RunNACCESS(command, naccess_data_dir)
    
    # parse outout
    new_asa = os.path.join(naccess_data_dir, "%s.asa" % naccess_data_base) 
    _ParseAsaFile(entity, new_asa, asa_atom)
      
    new_rsa = os.path.join(naccess_data_dir, "%s.rsa" % naccess_data_base) 
    _ParseRsaFile(entity, new_rsa, asa_abs, asa_rel)

  finally:
    # clean up
    if not keep_files:
      __CleanupFiles(naccess_data_dir)
  
  # sum up Asa for all atoms
  sasa = 0.0
  for a in entity.atoms:
    sasa += a.GetFloatProp(asa_atom, 0.0)

  return sasa



import unittest
from ost import stutil

class TestStUtils(unittest.TestCase):

  def setUp(self):
    self.data1 = [0,1,2,3,4,5,6,-5,-4,-3,-2,-1,1]
    self.data2 = [1,2,3,4,5,6,7,8,9,10,11,12,13]
    self.data3 = [0.1,0.5,0.7,2.4,0.5,4.1,0.9,-1.1,-0.5,0.7,-1.4,-7.5,8.5]
    self.data_short = [0,1]
    self.data_short2 = [0,1,2]

    self.mean = 0.5384615385
    self.mean2 = 0.6076923077
    self.median = 1
    self.median2 = 0.5
    self.median_short = 0.5
    self.median_short2 = 1.0
    self.stddev = 3.3192998478
    self.stddev2 = 3.4192805223
    self.minimum = -5
    self.minimum2 = -7.5
    self.maximum = 6
    self.maximum2 = 8.5
    self.correl = -0.39639313
    self.correl2 = -0.0619291504


  def isSimilar(self, value, reference, accuracy):
    if abs(value - reference) <= accuracy:
      return True
    return False

  def testMean(self):
    assert self.isSimilar(stutil.Mean(self.data1), self.mean, 0.001), \
           "Mean (%f) does not correspond to precalculated mean (%f)" % \
           (stutil.Mean(self.data1), self.mean)
    assert self.isSimilar(stutil.Mean(self.data3), self.mean2, 0.001), \
           "Mean (%f) does not correspond to precalculated mean (%f)" % \
           (stutil.Mean(self.data3), self.mean2)

  def testMedian(self):
    assert self.isSimilar(stutil.Median(self.data1), self.median, 0.001), \
           "Median (%f) does not correspond to precalculated median (%f)" % \
           (stutil.Median(self.data1), self.median)
    assert self.isSimilar(stutil.Median(self.data3), self.median2, 0.001), \
           "Median (%f) does not correspond to precalculated median (%f)" % \
           (stutil.Median(self.data3), self.median2)
    assert self.isSimilar(stutil.Median(self.data_short), self.median_short, 0.001), \
           "Median (%f) does not correspond to precalculated median (%f)" % \
           (stutil.Median(self.data_short), self.median_short)
    assert self.isSimilar(stutil.Median(self.data_short2), self.median_short2, 0.001), \
           "Median (%f) does not correspond to precalculated median (%f)" % \
           (stutil.Median(self.data_short2), self.median_short2)

  def testStddev(self):
    assert self.isSimilar(stutil.StdDev(self.data1), self.stddev, 0.001), \
           "StdDev (%f) does not correspond to precalculated StdDev (%f)" % \
           (stutil.StdDev(self.data1), self.stddev)
    assert self.isSimilar(stutil.StdDev(self.data3), self.stddev2, 0.001), \
           "StdDev (%f) does not correspond to precalculated StdDev (%f)" % \
           (stutil.StdDev(self.data3), self.stddev2)

  def testMinimum(self):
    assert self.isSimilar(stutil.Min(self.data1), self.minimum, 0.001), \
           "Minimum (%f) does not correspond to precalculated minimum (%f)" % \
           (stutil.Min(self.data1), self.minimum)
    assert self.isSimilar(stutil.Min(self.data3), self.minimum2, 0.001), \
           "Minimum (%f) does not correspond to precalculated minimum (%f)" % \
           (stutil.Min(self.data3), self.minimum2)

  def testMaximum(self):
    assert self.isSimilar(stutil.Max(self.data1), self.maximum, 0.001), \
           "Maximum (%f) does not correspond to precalculated maximum (%f)" % \
           (stutil.Max(self.data1), self.maximum)
    assert self.isSimilar(stutil.Max(self.data3), self.maximum2, 0.001), \
           "Maximum (%f) does not correspond to precalculated maximum (%f)" % \
           (stutil.Max(self.data3), self.maximum2)

  def testCorrel(self):
    assert self.isSimilar(stutil.Correl(self.data1, self.data2), self.correl, 0.001), \
           "Correl (%f) does not correspond to precalculated correl (%f)" % \
           (stutil.Correl(self.data1, self.data2), self.correl)
    assert self.isSimilar(stutil.Correl(self.data3, self.data2), self.correl2, 0.001), \
           "Correl (%f) does not correspond to precalculated correl (%f)" % \
           (stutil.Correl(self.data3, self.data2), self.correl2)

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()


#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
from .render_mode_widget import RenderModeWidget

#Trace Render Options
class TraceWidget(RenderModeWidget):
  def __init__(self, parent=None):
    RenderModeWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Trace"
     
    #Set Render Mode
    self.mode_ = gfx.RenderMode.TRACE
        
    min_arc_detail = 1
    max_arc_detail = 20
    
    min_width = 0.1
    max_width = 2.5
    max_tube_width= 2.0
    
    #########UI##########
            
    #Arc Label
    arc_label = QtWidgets.QLabel("Arc Detail")
    
    self.arc_spinbox_ = QtWidgets.QSpinBox()
    self.arc_spinbox_.setRange(min_arc_detail, max_arc_detail)
        
    #Tube Radius
    radius_tube_label = QtWidgets.QLabel("Radius")
    
    self.width_tube_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.width_tube_spinbox_.setRange(min_width, max_tube_width)
    self.width_tube_spinbox_.setDecimals(1)
    self.width_tube_spinbox_.setSingleStep(0.1)
    
    self.width_tube_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.width_tube_slider_.setRange(min_width*10.0, max_tube_width*10.0)
    self.width_tube_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.width_tube_slider_.setTickInterval(1)

    grid = QtWidgets.QGridLayout()
    grid.addWidget(arc_label,3,0,1,3)
    grid.addWidget(self.arc_spinbox_,3,4,1,1)
    
    grid.addWidget(radius_tube_label, 5, 0, 1, 1)
    grid.addWidget(self.width_tube_slider_, 5, 1, 1, 3)
    grid.addWidget(self.width_tube_spinbox_, 5, 4, 1, 1)
        
    grid.setRowStretch(15,1)
    self.setLayout(grid)
    
    self.arc_spinbox_.valueChanged.connect(self.UpdateArcDetail)
    self.width_tube_spinbox_.valueChanged.connect(self.UpdateTubeRadius)
    self.width_tube_slider_.valueChanged.connect(self.UpdateSliderTubeRadius)
    
    self.setMinimumSize(250,60) #2*30
    ########/UI########
  def UpdateGui(self,options):
    self.arc_spinbox_.setValue(options.GetArcDetail())
    
    self.UpdateTubeRadiusGui(options.GetTubeRadius())
    
  def UpdatePolyMode(self, value):
    self.GetOptions().SetPolyMode(value)
    self.ApplyOptions()
        
  def UpdateArcDetail(self, value):
    self.GetOptions().SetArcDetail(value)
    self.ApplyOptions()
    
  def UpdateTubeRadius(self, value):
    self.GetOptions().SetTubeRadius(value)
    
  def UpdateSliderTubeRadius(self, value):
    self.GetOptions().SetTubeRadius(value/10.0)

  def UpdateTubeRadiusGui(self,value):
    value = round(value, 2)
    if(abs(value*10.0 - self.width_tube_slider_.value())>=self.width_tube_slider_.singleStep()):
      self.width_tube_slider_.setValue(value*10.0)
    if(abs(value - self.width_tube_spinbox_.value())>=self.width_tube_spinbox_.singleStep()):
      self.width_tube_spinbox_.setValue(value)
  
  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import info
from ost import gfx

class RenderOp:
  RENDERMODE_ATTRIBUTE_NAME = "RenderMode"
  KEEP_ATTRIBUTE_NAME = "Keep"
  FLAGS_ATTRIBUTE_NAME = "Flags"
    
  def __init__(self, render_mode, selection, flags, keep=False):
    self.render_mode_ = render_mode
    self.selection_ = selection
    self.keep_ = keep
    self.flags_ = flags

  def GetName(self):
    return "Render mode: %s"%str(self.GetRenderMode())

  def SetRenderMode(self, render_mode):
    self.render_mode_ = render_mode
    
  def GetRenderMode(self):
    return self.render_mode_
 
  def SetSelection(self, selection):
    self.selection_ = selection
    
  def GetSelection(self):
    return self.selection_

  def SetSelectionFlags(self, flags):
    self.flags_ = flags
    
  def GetSelectionFlags(self):
    return self.flags_

  def SetKeep(self, keep):
    self.keep_ = keep

  def IsKept(self):
    return self.keep_

  def ApplyOn(self, entity):
    if (entity is not None) and isinstance(entity, gfx.Entity):
      entity.SetRenderMode(self.GetRenderMode(),entity.view.Select(self.GetSelection(),self.GetSelectionFlags()),self.IsKept())
  
  def ToInfo(self,group):
    group.SetAttribute(RenderOp.RENDERMODE_ATTRIBUTE_NAME, str(self.GetRenderMode().name))
    group.SetAttribute(RenderOp.KEEP_ATTRIBUTE_NAME, str(int(self.IsKept())))
    group.SetAttribute(RenderOp.FLAGS_ATTRIBUTE_NAME, str(self.GetSelectionFlags()))
    group.SetTextData(str(self.GetSelection()))
    
  @staticmethod
  def FromInfo(group):
    render_op = None
    if (group.HasAttribute(RenderOp.RENDERMODE_ATTRIBUTE_NAME) 
    and  group.HasAttribute(RenderOp.KEEP_ATTRIBUTE_NAME)):
      render_mode = getattr(gfx.RenderMode, group.GetAttribute(RenderOp.RENDERMODE_ATTRIBUTE_NAME))
      keep = bool(int(group.GetAttribute(RenderOp.KEEP_ATTRIBUTE_NAME)))
      flags=0
      if group.HasAttribute(RenderOp.FLAGS_ATTRIBUTE_NAME):
        flags = int(group.GetAttribute(RenderOp.FLAGS_ATTRIBUTE_NAME))
      selection = group.GetTextData()
      render_op = RenderOp(render_mode,selection,flags,keep)
    return render_op



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets


#Tube Render Options
class RenderModeWidget(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.options_ = None
    self.entities_ = set()

  def GetText(self):
    raise NotImplementedError("Subclasses must define GetText()")
  
  def GetRenderMode(self):
    raise NotImplementedError("Subclasses must define GetRenderMode()")
    
  def UpdateGui(self, options):
    pass
  
  def Update(self):
    self.entities_.clear()
    
    scene_selection = gui.SceneSelection.Instance()
    if scene_selection.GetActiveNodeCount() == 0 and scene_selection.GetActiveViewCount() == 0:
      self.setEnabled(False)
      return
    
    if scene_selection.GetActiveNodeCount() > 0 :
      for i in range(0,scene_selection.GetActiveNodeCount()):
        entity = scene_selection.GetActiveNode(i)
        if isinstance(entity, gfx.Entity):
          self.entities_.add(entity)
        else:
          self.setEnabled(False)
          return

    if scene_selection.GetActiveViewCount() > 0 :
      entity = scene_selection.GetViewEntity()
      self.entities_.add(entity)
    
    if len(self.entities_)>0:
      entity = self.entities_.pop()
      self.options_=entity.GetOptions(self.GetRenderMode())
      self.UpdateGui(self.options_)
      QtWidgets.QWidget.setEnabled(self,True)
    else:
      QtWidgets.QWidget.setEnabled(self,False)
    
  def ApplyOptions(self):
    for entity in self.entities_:
      entity.ApplyOptions(self.GetRenderMode(), self.GetOptions())

  def GetOptions(self):
    return self.options_



model_file='sdh.pdb'
e=io.LoadEntity(model_file)


def StripSidechain(edi, residue):
  atoms = residue.GetAtomList()
  for atom in atoms:
    if atom.name not in ['CA', 'C', 'N', 'O']:
      edi.DeleteAtom(atom)

def StripSidechains(entity):
  edi=entity.EditXCS(mol.EditMode.BUFFERED_EDIT)
  for c in e.GetChainList():
    residues=c.GetResidueList()
    for residue in residues:
      StripSidechain(edi, residue)

StripSidechains(e)
g=gfx.Entity('bare backbone', e)
scene.Add(g)
scene.CenterOn(g)





eh=io.LoadPDB("1crn",remote=True)

scene.Add(gfx.Entity("simple_default",gfx.SIMPLE,eh))
scene.Add(gfx.Entity("simple_mod",gfx.SIMPLE,eh))
scene["simple_mod"].ColorBy("abfac",gfx.Gradient([gfx.BLUE,gfx.RED]))
scene["simple_mod"].simple_options.line_width=3.5
scene["simple_mod"].simple_options.aa_lines=True

scene.Add(gfx.Entity("custom_default",gfx.CUSTOM,eh))
scene.Add(gfx.Entity("custom_mod",gfx.CUSTOM,eh))
scene["custom_mod"].custom_options.bond_rad=0.45
scene["custom_mod"].custom_options.sphere_rad=0.55
scene["custom_mod"].SetColor(gfx.ORANGE,"ele=C")

scene.Add(gfx.Entity("cpk_default", gfx.CPK,eh))
scene["cpk_default"].cpk_options.sphere_detail=2 # should have no influence
scene.Add(gfx.Entity("cpk_mod", gfx.CPK,eh))
scene["cpk_mod"].cpk_options.sphere_mode=0
scene["cpk_mod"].cpk_options.sphere_detail=2

scene.Add(gfx.Entity("ltrace_default", gfx.LINE_TRACE,eh))
scene.Add(gfx.Entity("ltrace_mod", gfx.LINE_TRACE,eh))
scene["ltrace_mod"].ColorBy("rnum",gfx.Gradient([gfx.PURPLE,gfx.YELLOW,gfx.CYAN]))
scene["ltrace_mod"].line_trace_options.aa_lines=True
scene["ltrace_mod"].line_trace_options.line_width=3.5

scene.Add(gfx.Entity("trace_default", gfx.TRACE,eh))
scene.Add(gfx.Entity("trace_mod", gfx.TRACE,eh))
scene["trace_mod"].ColorBy("rnum",gfx.Gradient([gfx.RED,gfx.GREEN,gfx.BLUE]))
scene["trace_mod"].trace_options.arc_detail=6
scene["trace_mod"].trace_options.tube_radius=1.3

scene.Add(gfx.Entity("tube_default", gfx.TUBE,eh))
eh2=eh.Copy()
scene.Add(gfx.Entity("tube_mod", gfx.TUBE,eh2))
bfmin=1e-6
bfmax=-1e-6
for a in eh2.Select("peptide=1 and aname=CA").atoms:
  bfmin=min(bfmin,a.b_factor)
  bfmax=max(bfmax,a.b_factor)
for a in eh2.Select("peptide=1 and aname=CA").atoms:
  a.handle.SetFloatProp("trace_rad",5.0*min(1.0,max(0.0,(a.b_factor-bfmin)/(bfmax-bfmin)))+0.3)
scene["tube_mod"].ColorBy("rbfac",gfx.Gradient([gfx.BLUE,gfx.RED]))
scene["tube_mod"].tube_options.spline_detail=8
scene["tube_mod"].tube_options.arc_detail=8
scene["trace_mod"].tube_options.tube_radius=1.0

scene.Add(gfx.Entity("hsc_default", gfx.HSC,eh))
scene.Add(gfx.Entity("hsc_mod", gfx.HSC,eh))
scene["hsc_mod"].SetColor(gfx.PURPLE,"rtype=H")
scene["hsc_mod"].SetColor(gfx.ORANGE,"rtype=E")
scene["hsc_mod"].cartoon_options.helix_width=1.3
scene["hsc_mod"].cartoon_options.helix_thickness=0.6
scene["hsc_mod"].cartoon_options.strand_width=1.6
scene["hsc_mod"].cartoon_options.strand_thickness=0.4
scene["hsc_mod"].cartoon_options.tube_radius=0.5



scene.Match("*").Hide()




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
from .render_mode_widget import RenderModeWidget

#Custom Render Options
class CustomWidget(RenderModeWidget):
  def __init__(self, parent=None):
    RenderModeWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Ball & Stick"
    
    #Defaults
    min_sphere_detail = 1
    max_sphere_detail = 12
    
    min_arc_detail = 1
    max_arc_detail = 12
    
    min_rad = 0.05
    max_bond_rad = 5.0
    max_sphere_rad = 5.0
    
    
    
    #Set Render Mode
    self.mode_ = gfx.RenderMode.CUSTOM
        
    #Create Ui elements
    
    self.sphere_spinbox_ = QtWidgets.QSpinBox()
    self.sphere_spinbox_.setRange(min_sphere_detail, max_sphere_detail)
    
    self.arc_spinbox_ = QtWidgets.QSpinBox()
    self.arc_spinbox_.setRange(min_arc_detail, max_arc_detail)
  
    #Bond Radius
    radius_bond_label = QtWidgets.QLabel("Bond Radius")
    
    self.radius_bond_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.radius_bond_spinbox_.setRange(min_rad, max_bond_rad)
    self.radius_bond_spinbox_.setDecimals(2)
    self.radius_bond_spinbox_.setSingleStep(0.05)
    
    self.radius_bond_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.radius_bond_slider_.setRange(min_rad*100.0, max_bond_rad*100.0)
    self.radius_bond_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.radius_bond_slider_.setTickInterval(5)
    
    
    #Sphere Radius
    radius_sphere_label = QtWidgets.QLabel("Sphere Radius")
    
    self.radius_sphere_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.radius_sphere_spinbox_.setRange(min_rad, max_sphere_rad)
    self.radius_sphere_spinbox_.setDecimals(2)
    self.radius_sphere_spinbox_.setSingleStep(0.05)
    
    self.radius_sphere_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.radius_sphere_slider_.setRange(min_rad*100.0, max_sphere_rad*100.0)
    self.radius_sphere_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.radius_sphere_slider_.setTickInterval(5)
    
    custom_label = QtWidgets.QLabel(self.text_)
    font = custom_label.font()
    font.setBold(True)
    
    sphere_label = QtWidgets.QLabel("Sphere Detail")
    arc_label = QtWidgets.QLabel("Arc Detail")
    grid = QtWidgets.QGridLayout()
    grid.addWidget(custom_label,0,0,1,1)
    grid.addWidget(sphere_label, 1, 0, 1, 3)
    grid.addWidget(self.sphere_spinbox_, 1, 2, 1, 1)
    grid.addWidget(arc_label,2,0,1,3)
    grid.addWidget(self.arc_spinbox_,2,2,1,1)
    
    grid.addWidget(radius_bond_label, 3, 0, 1, 1)
    grid.addWidget(self.radius_bond_slider_, 3, 1, 1, 3)
    grid.addWidget(self.radius_bond_spinbox_, 3, 4, 1, 1)
    
    grid.addWidget(radius_sphere_label, 4, 0, 1, 1)
    grid.addWidget(self.radius_sphere_slider_, 4, 1, 1, 3)
    grid.addWidget(self.radius_sphere_spinbox_, 4, 4, 1, 1)
    
    grid.setRowStretch(3,1)
    self.setLayout(grid)
  
    self.sphere_spinbox_.valueChanged.connect(self.UpdateSphereDetail)
    self.arc_spinbox_.valueChanged.connect(self.UpdateArcDetail)
    
    self.radius_bond_spinbox_.valueChanged.connect(self.UpdateBondRadius)
    self.radius_bond_slider_.valueChanged.connect(self.UpdateSliderBondRadius)
    
    self.radius_sphere_spinbox_.valueChanged.connect(self.UpdateSphereRadius)
    self.radius_sphere_slider_.valueChanged.connect(self.UpdateSliderSphereRadius)
    
    self.setMinimumSize(250,150)
    
  def UpdateSphereDetail(self, value):
    self.GetOptions().SetSphereDetail(value)
    self.ApplyOptions()

  def UpdateArcDetail(self, value):
    self.GetOptions().SetArcDetail(value)
    self.ApplyOptions()

  def UpdateBondRadius(self, value):
    self.GetOptions().SetBondRad(value)
    if(self.GetOptions().GetSphereRad()<self.GetOptions().GetBondRad()):
      self.GetOptions().SetSphereRad(value)
    self.ApplyOptions()
    
  def UpdateSliderBondRadius(self, value):
    self.GetOptions().SetBondRad(value/100.0)
    self.ApplyOptions()
    
  def UpdateSphereRadius(self, value):
    self.GetOptions().SetSphereRad(value)
    if(self.GetOptions().GetSphereRad()<self.GetOptions().GetBondRad()):
      self.GetOptions().SetBondRad(value)
    self.ApplyOptions()
    
  def UpdateSliderSphereRadius(self, value):
    self.GetOptions().SetSphereRad(value/100.0)
    self.ApplyOptions()

  def UpdateSphereRadiusGui(self,value):
    value = round(value, 2)
    if(abs(value*100.0 - self.radius_sphere_slider_.value())>=self.radius_sphere_slider_.singleStep()):
      self.radius_sphere_slider_.setValue(value*100.0)
    if (abs(value - self.radius_sphere_spinbox_.value())>=self.radius_sphere_spinbox_.singleStep()):
      self.radius_sphere_spinbox_.setValue(value)

  def UpdateBondRadiusGui(self,value):
    value = round(value, 2)
    if(abs(value*100.0 - self.radius_bond_slider_.value())>=self.radius_bond_slider_.singleStep()):
      self.radius_bond_slider_.setValue(value*100.0)
    if (abs(value - self.radius_bond_spinbox_.value())>=self.radius_bond_spinbox_.singleStep()):
      self.radius_bond_spinbox_.setValue(value)

  def UpdateGui(self,options):
    self.sphere_spinbox_.setValue(options.GetSphereDetail())    
    self.arc_spinbox_.setValue(options.GetArcDetail())
    
    self.UpdateBondRadiusGui(options.GetBondRad())
    self.UpdateSphereRadiusGui(options.GetSphereRad())

  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



import os.path
from ._ost_mol_alg import *
from ost.mol.alg.superpose import *
import ost.mol.alg.trajectory_analysis
import ost.mol.alg.structure_analysis
import ost.mol.alg.helix_kinks
import ost.mol.alg.hbond

# Fills a list of reference clashing distances from a file (requires a path to the file)
def FillClashingDistancesFromFile(filename):
  fh=open(filename,'r')
  lines=fh.readlines()
  fh.close()
  return FillClashingDistances(lines)

# Fills a list of bond stereo-chemical statistics from a file (requires a path to the file)
def FillBondStereoChemicalParamsFromFile(filename):
  fh=open(filename,'r')
  lines=fh.readlines()
  fh.close()
  return FillBondStereoChemicalParams("Bond",lines)

# Fills a list of angle stereo-chemical statistics from a file (requires a path to the file)
def FillAngleStereoChemicalParamsFromFile(filename):
  fh=open(filename,'r')
  lines=fh.readlines()
  fh.close()
  return FillAngleStereoChemicalParams("Angle",lines)

# Returns the default list of reference clashing distances (from the default OpenStructure parameter file)
def DefaultClashingDistances():
  shared_path=ost.GetSharedDataPath()	
  filename=os.path.join(shared_path,'stereo_chemical_props.txt')
  fh=open(filename,'r')
  lines=fh.readlines()
  fh.close()
  return FillClashingDistances(lines)

# Returns the default list of bond stereo-chemical statistics (from the default OpenStructure parameter file)
def DefaultBondStereoChemicalParams():
  shared_path=ost.GetSharedDataPath()	
  filename=os.path.join(shared_path,'stereo_chemical_props.txt')
  fh=open(filename,'r')
  lines=fh.readlines()
  fh.close()
  return FillStereoChemicalParams("Bond",lines)

# Returns the default list of angle stereo-chemical statistics (from the default OpenStructure parameter file)
def DefaultAngleStereoChemicalParams():
  shared_path=ost.GetSharedDataPath()	
  filename=os.path.join(shared_path,'stereo_chemical_props.txt')
  fh=open(filename,'r')
  lines=fh.readlines()
  fh.close()
  return FillStereoChemicalParams("Angle",lines)




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

import sys
from ost import mol
from ost import gui
from ost import gfx
try: 
  from ost import img
  _img_present=True
except ImportError:
  _img_present=False
  pass

from PyQt5 import QtCore, QtWidgets, QtWidgets
from .scene_selection_helper import SelHelper
from .gradient_editor_widget import GradientEditor
from .uniform_color_widget import UniformColorWidget
from .combo_options_widget import ComboOptionsWidget

class ColorOptionsWidget(ComboOptionsWidget):
  def __init__(self, parent=None):
    ComboOptionsWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Color Options"
    conly_label_ = QtWidgets.QLabel('carbons only')
    self.conly_box_ = QtWidgets.QCheckBox()
    
    #Add options to menu
    self.entity_widgets_ = list()
    self.entity_widgets_.append(["Color by Element", ByElementWidget("Color by Element", self)])
    self.entity_widgets_.append(["Color by Chain", ByChainWidget("Color by Chain", self)])
    self.entity_widgets_.append(["Color by Entity", ByEntityWidget("Color by Entity", self)])
    self.entity_widgets_.append(["Color by Property", GradientEditor(self)])
    self.entity_widgets_.append(["Uniform",UniformColorWidget(self)])
  
    self.img_widgets_ = list()
    self.img_widgets_.append(["Uniform",UniformColorWidget()])

    qw = QtWidgets.QWidget(self)
    gl = QtWidgets.QGridLayout(qw)
    gl.addWidget(self.conly_box_, 0, 0, 1, 1)
    gl.addWidget(conly_label_, 0, 1, 1, 4)
    self.grid_layout_.addWidget(qw, 2, 0, 1, 1)

    self.setMinimumSize(250,200)
    
  def OnComboChange(self, item):
    scene_selection = gui.SceneSelection.Instance()
    if hasattr(item,"PrepareColoring"):
      item.PrepareColoring()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      item.ChangeColor(node)
    
    if(scene_selection.GetActiveViewCount() > 0):
      entity = scene_selection.GetViewEntity()
      view = scene_selection.GetViewUnion()
      item.ChangeViewColor(entity,view)

    self.DoResize()
    
    
  def Update(self):
    
    ComboOptionsWidget.setEnabled(self,True)
    
    if SelHelper().CheckAllFlags(SelHelper.NO_SELECTION):
      ComboOptionsWidget.setEnabled(self,False)
      return
    
    for w in self.entity_widgets_:
      self.RemoveWidget(w[0])
    for w in self.img_widgets_:
      self.RemoveWidget(w[0])
    
    
    if SelHelper().CheckFlags(SelHelper.HAS_IMG | SelHelper.IS_ONE_TYPE):
      for w in self.img_widgets_:
        self.AddWidget(w[0], w[1])
    elif SelHelper().CheckMinOneFlag(SelHelper.HAS_ENTITY| SelHelper.HAS_VIEW| SelHelper.HAS_SURFACE) and SelHelper().CheckNotFlags(SelHelper.HAS_IMG):
      for w in self.entity_widgets_:
        self.AddWidget(w[0], w[1])
    else:
      ComboOptionsWidget.setEnabled(self,False)
      return
    
    self.GetCurrentWidget().Update()
    
  def GetText(self):
    return self.text_

  def GetCarbonsOnly(self):
    return self.conly_box_.isChecked()
  

class ByElementWidget(QtWidgets.QWidget):
  def __init__(self, text, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.parent_ = parent

    
    #Title
    self.text_ = text
    
    #UI
    text_label = QtWidgets.QLabel(text)
    font = text_label.font()
    font.setBold(True)
    
    grid = QtWidgets.QGridLayout()
    grid.addWidget(text_label,0,0,1,1)
    grid.addWidget(QtWidgets.QLabel("No Settings available"), 1, 0, 1, 3)
    grid.setRowStretch(2,1)
    self.setLayout(grid)
    self.setMinimumSize(250,60)
    
  def Update(self):
   pass #Do Nothing
    
  def ChangeColor(self, node):
    if isinstance(node, gfx.Entity):
      node.CleanColorOps()
      if self.parent_.GetCarbonsOnly():
        node.ColorByElement("ele=C")
      else:
        node.ColorByElement()
      
  def ChangeViewColor(self, entity, view):
    if isinstance(entity, gfx.Entity) and isinstance(view, mol.EntityView):
      if self.parent_.GetCarbonsOnly():
        beco=gfx.ByElementColorOp(mol.QueryViewWrapper(mol.Query("ele=C"), view))
      else:
        beco=gfx.ByElementColorOp(mol.QueryViewWrapper(view))
      entity.Apply(beco)
      
  def GetText(self):
    return self.text_
  
  
class ByChainWidget(QtWidgets.QWidget):
  def __init__(self, text, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.parent_ = parent
    
    #Title
    self.text_ = text
    
    #UI
    text_label = QtWidgets.QLabel(text)
    font = text_label.font()
    font.setBold(True)
    
    grid = QtWidgets.QGridLayout()
    grid.addWidget(text_label,0,0,1,1)
    grid.setRowStretch(2,1)
    self.setLayout(grid)
    self.setMinimumSize(250,60)
    
  def Update(self):
   pass #Do Nothing
    
  def ChangeColor(self, node):
    if isinstance(node, gfx.Entity):
      node.CleanColorOps()
      if self.parent_.GetCarbonsOnly():
        node.ColorByChain('ele=C')
      else:
        node.ColorByChain()
      
  def ChangeViewColor(self, entity, view):
    if isinstance(entity, gfx.Entity) and isinstance(view, mol.EntityView):
      if self.parent_.GetCarbonsOnly():
        bco=gfx.ByChainColorOp(mol.QueryViewWrapper(mol.Query("ele=C"),view))
      else:
        bco=gfx.ByChainColorOp(mol.QueryViewWrapper(view))
      entity.Apply(bco)
      
  def GetText(self):
    return self.text_

class ByEntityWidget(QtWidgets.QWidget):
  def __init__(self, text, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.parent_ = parent
    
    #Title
    self.text_ = text
    
    #UI
    text_label = QtWidgets.QLabel(text)
    font = text_label.font()
    font.setBold(True)
    
    grid = QtWidgets.QGridLayout()
    grid.addWidget(text_label,0,0,1,1)
    grid.setRowStretch(2,1)
    self.setLayout(grid)
    self.setMinimumSize(250,60)
    
    self.gradient_ = gfx.Gradient("RAINBOW")
    
  def Update(self):
   pass #Do Nothing
    
  def PrepareColoring(self):
    scene_selection = gui.SceneSelection.Instance()
    entity_count = scene_selection.GetActiveNodeCount()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      if i<=0:
        color=self.gradient_.GetColorAt(0.0)
      else:
        color=self.gradient_.GetColorAt(float(i) / entity_count)
      node = scene_selection.GetActiveNode(i)
      if self.parent_.GetCarbonsOnly():
        node.SetColor(color, 'ele=C')
      else:
        node.SetColor(color)
  
  def ChangeColor(self, node):
    pass
     
  def ChangeViewColor(self, entity, view):
    if isinstance(entity, gfx.Entity) and isinstance(view, mol.EntityView):
      if self.parent_.GetCarbonsOnly():
        bco=gfx.ByChainColorOp(mol.QueryViewWrapper(mol.Query("ele=C"),view))
      else:
        bco=gfx.ByChainColorOp(mol.QueryViewWrapper(view))
      entity.Apply(bco)
      
  def GetText(self):
    return self.text_



from ost.mol import mm
from PyQt5 import QtCore

"""
MM - Demo

!!!Requires OpenStructure to be compiled with MM-Support!!!

This demo should show the setup of a simple simulation using implicit solvent.
The script can directly be started from within DNG.
"""

class Anim(QtCore.QTimer):
    def __init__(self,sim,ent,go):
        QtCore.QTimer.__init__(self)
        self.ent=ent
        self.sim=sim
        self.go = go
        self.ed = ent.EditXCS(ost.mol.BUFFERED_EDIT)
        self.timeout.connect(self.OnTimer)

        
    def OnTimer(self):
        self.sim.Steps(5)
        self.sim.UpdatePositions()
        self.go.UpdatePositions()


prot=ost.mol.CreateEntityFromView(io.LoadPDB('1CRN.pdb').Select('peptide=true'),True)

settings = mm.Settings()

settings.integrator = mm.LangevinIntegrator(310,1,0.002)
settings.add_gbsa = True
settings.forcefield = mm.LoadCHARMMForcefield()
settings.platform = mm.Platform.CPU
settings.nonbonded_method = mm.NonbondedMethod.CutoffNonPeriodic

sim = mm.Simulation(prot,settings)
sim.ApplyLBFGS(tolerance = 1.0, max_iterations = 200)
sim.UpdatePositions()

ent = sim.GetEntity()
go = gfx.Entity("crambin",ent)
scene.Add(go)

anim = Anim(sim,ent,go)
anim.start(1)




"""
Interface to HB+ command line program

Author: Marco Biasini
"""
import tempfile
from ost import settings
import subprocess
import re
import os
import shutil
from ost import io
from ost import mol

def _LocateHBPlus(hbplus_bin):
  return settings.Locate('hbplus', explicit_file_name=hbplus_bin,
                         env_name="HBPLUS")

class HBond:
  def __init__(self, donor, acceptor):
    self.donor=donor
    self.acceptor=acceptor

def _ParseOutput(ent, output):
  hbonds=[]
  # skip header
  lines=list(output)
  for index, line in enumerate(lines):
    if line.startswith('<---DONOR---> <-ACCEPTOR-->'):
      lines=lines[index+4:]
      break
  for line in lines:
    if len(line.strip())==0:
      continue
    don_chain=line[0]
    don_rnum=int(line[1:5])
    don_ins_c=line[5]
    don_atom=line[9:13]
    if don_chain=='-':
      don_chain=' '
    if don_ins_c=='-':
      don_ins_c='\0'
    acc_chain=line[14]
    acc_rnum=int(line[15:19])
    acc_ins_c=line[19]
    acc_atom=line[24:28]
    if acc_chain=='-':
      acc_chain=' '
    if acc_ins_c=='-':
      acc_ins_c='\0'
    donor=ent.FindAtom(don_chain, mol.ResNum(don_rnum, don_ins_c),
                       don_atom.strip())
    acc=ent.FindAtom(acc_chain, mol.ResNum(acc_rnum, acc_ins_c),
                     acc_atom.strip())                       
    assert donor.IsValid()
    assert acc.IsValid()
    hbonds.append(HBond(donor, acc))
  return hbonds

def HBondList(ent, hbplus_bin=None):
  """
  returns a list of HBonds found in the given entity (handle or view)
  """
  full_bin=_LocateHBPlus(hbplus_bin)
  temp_d=tempfile.mkdtemp(prefix='hbplus_')
  file_name=os.path.join(temp_d, 'ent.pdb')
  io.SaveEntity(ent, file_name)
  hb_proc=subprocess.Popen(full_bin, shell=True, stdout=subprocess.PIPE,
                           stdin=subprocess.PIPE)
  hb_proc.stdin.write(('%s\n' % temp_d).encode())
  hb_proc.stdin.write(('%s\n\n' % file_name).encode())
  stdout,_ = hb_proc.communicate()

  for line in stdout.decode().splitlines():
    match=re.match(r'Configured for (\d+) atoms and\s+(\d+) residues\.', line)
    if match:
      assert ent.atom_count<int(match.group(1))
      assert ent.residue_count<int(match.group(2))
  hb_out=open(os.path.join(temp_d, 'ent.hb2'), 'r')
  hbonds=_ParseOutput(ent, hb_out)
  hb_out.close()
  shutil.rmtree(temp_d)
  return hbonds

def HBondScore(ent1, ent2, hbplus_bin=None):
  """
  returns percentage of hydrogen bonds in ent1 that are also present in ent2 in
  the range of 0 to 1.
  
  this function is slow as hell, and could be improved drastically by first
  sorting the hydrogen bond lists by donor residue number.
  """

  hbonds_a=HBondList(ent1, hbplus_bin=hbplus_bin)
  hbonds_b=HBondList(ent2, hbplus_bin=hbplus_bin)
  matching=0
  for a in hbonds_a:
    found=False
    for b in hbonds_b:
      acc_names_match=b.acceptor.name==a.acceptor.name
      don_names_match=b.donor.name==a.donor.name
      don_num=b.donor.residue.number==a.donor.residue.number
      acc_num=b.acceptor.residue.number==a.acceptor.residue.number
      if acc_names_match and don_names_match and acc_num and don_num:
        found=True
        break
    if found:
      matching+=1
  if len(hbonds_a)>0:
    return float(matching)/len(hbonds_a)
  else:
    return 0.0

if __name__=='__main__':
  print('HBond Score:', HBondScore(io.LoadPDB(sys.argv[1]), 
                                   io.LoadPDB(sys.argv[2])))





class TableSelector:

  def __init__(self, col_types, col_names, query):

    self.col_types=col_types
    self.col_names=col_names
    self.query=query
    
    self.valid_operators=dict()

    self.valid_operators['and']={'type':'boolean','precedence':5}
    self.valid_operators['or']={'type':'boolean','precedence':6}

    self.valid_operators['!']={'type':'comparison','precedence':4,'extension':'!='}
    self.valid_operators['!=']={'type':'comparison','precedence':4,'extension':None}
    self.valid_operators['=']={'type':'comparison','precedence':4,'extension':'=='}
    self.valid_operators['==']={'type':'comparison','precedence':4,'extension':None}
    self.valid_operators['<=']={'type':'comparison','precedence':3,'extension':None}
    self.valid_operators['>=']={'type':'comparison','precedence':3,'extension':None}
    self.valid_operators['>']={'type':'comparison','precedence':3,'extension':'>='}
    self.valid_operators['<']={'type':'comparison','precedence':3,'extension':'<='}

    self.valid_operators['+']={'type':'arithmetic','precedence':2}
    self.valid_operators['-']={'type':'arithmetic','precedence':2}
    self.valid_operators['/']={'type':'arithmetic','precedence':1}
    self.valid_operators['*']={'type':'arithmetic','precedence':1}

    self.valid_operators['(']={'type':'left_bracket','precedence':float('NaN')}
    self.valid_operators['[']={'type':'left_bracket','precedence':float('NaN')}
    self.valid_operators['{']={'type':'left_bracket','precedence':float('NaN')}
    self.valid_operators[')']={'type':'right_bracket','precedence':float('NaN')}
    self.valid_operators[']']={'type':'right_bracket','precedence':float('NaN')}
    self.valid_operators['}']={'type':'right_bracket','precedence':float('NaN')}

    self.split_expression=self._ExpressionLexer(self.query)
    self.parsed_expression=self._ParseExpression(self.split_expression)
    self.rpn_expression=self._ShuntingYard(self.parsed_expression)

    self.tab_indices=list()
    self.exp_indices=list()

    #extract column indices
    for i, exp in enumerate(self.rpn_expression):
      if exp in self.col_names:
        self.tab_indices.append(self._GetIndex(exp))
        self.exp_indices.append(i)

  def EvaluateRow(self,row):
      for ti, ei in zip(self.tab_indices, self.exp_indices):
        #check for NaN
        if row[ti]!=row[ti]:
          self.rpn_expression[ei]=None
        else:
          self.rpn_expression[ei] = row[ti]
      if self._EvaluateRPN(list(self.rpn_expression)):
        return True
      return False

  def _GetIndex(self, col):
    if col not in self.col_names:
      raise ValueError('Table Selector has no column named "%s"' % col)
    return self.col_names.index(col)
    
  def _EvaluateAnd(self, lhs, rhs):
    return lhs==True and rhs==True

  def _EvaluateOr(self, lhs, rhs):
    return lhs==True or rhs==True

  def _EvaluateEqual(self, lhs, rhs):
    return lhs==rhs

  def _EvaluateNonEqual(self, lhs, rhs):
    return lhs!=rhs

  def _EvaluateLower(self, lhs, rhs):
    if lhs==None or rhs==None:
      return False
    return lhs<rhs

  def _EvaluateGreater(self, lhs, rhs):
    if lhs==None or rhs==None:
      return False
    return lhs>rhs

  def _EvaluateLowerEqual(self, lhs, rhs):
    if lhs==None or rhs==None:
      return False
    return lhs<=rhs

  def _EvaluateGreaterEqual(self, lhs, rhs):
    if lhs==None or rhs==None:
      return False
    return lhs>=rhs

  def _EvaluateAdd(self, lhs, rhs):
    if lhs==None or rhs==None:
      return None
    return lhs+rhs

  def _EvaluateSubtract(self, lhs, rhs):
    if lhs==None or rhs==None:
      return None
    return lhs-rhs

  def _EvaluateMultiply(self, lhs, rhs):
    if lhs==None or rhs==None:
      return None
    return lhs*rhs

  def _EvaluateDivide(self, lhs, rhs):
    if lhs==None or rhs==None:
      return None
    return lhs/rhs


  def _EvaluateOperator(self, op, lhs, rhs):

    #this function assumes, that all NaN values have been replaced by None!

    if op=='+':
      return self._EvaluateAdd(lhs, rhs)
    elif op=='-':
      return self._EvaluateSubtract(lhs, rhs)
    elif op=='/':
      return self._EvaluateDivide(lhs, rhs)
    elif op=='*':
      return self._EvaluateMultiply(lhs, rhs)
    elif op=='and':
      return self._EvaluateAnd(lhs, rhs)
    elif op=='or':
      return self._EvaluateOr(lhs, rhs)
    elif op=='=' or op=='==':
      return self._EvaluateEqual(lhs, rhs)
    elif op=='!=' or op=='!':
      return self._EvaluateNonEqual(lhs, rhs)
    elif op=='<':
      return self._EvaluateLower(lhs, rhs)
    elif op=='>':
      return self._EvaluateGreater(lhs, rhs)
    elif op=='<=':
      return self._EvaluateLowerEqual(lhs, rhs)
    elif op=='>=':
      return self._EvaluateGreaterEqual(lhs, rhs)

    else:
      raise ValueError('Unknown operator: '+op)

  def _EvaluateRPN(self, RPNExp):
    #Evaluates the reverse polish notation
    stack=list()
    while True:
      if len(RPNExp)==0:
        break
      exp=RPNExp.pop(0)
      if exp in self.valid_operators:
        if len(stack)<2:
          raise ValueError('Cannot evaluate operator on less than two operands!')
        rhs=stack.pop()
        lhs=stack.pop()
        result=self._EvaluateOperator(exp, lhs, rhs)
        if result==None:
          return False
        stack.append(result)
      else:
        stack.append(exp)
    if len(stack)>1:
      raise ValueError('Too many operands for given operators!')
    return stack.pop()

  def _ShuntingYard(self, split_expression):
    #Creates the so called reverse polish notation out of the expression parser output.
    #note, that there won't be parenthesis anymore and potential parenthesis
    #mismatches get recognized.
    #The shunting yard algorithm from dijkstra gets used.

    output_stack=list()
    operator_stack=list()

    while True:
      if len(split_expression)==0:
        while True:
          if len(operator_stack)==0:
            break
          if self.valid_operators[operator_stack[-1]]['type'] in ['left_bracket','right_bracket']:
            raise ValueError('Parenthesis mismatch!')
          output_stack.append(operator_stack.pop())
        break

      exp=split_expression.pop(0)

      if exp in self.valid_operators:
        if self.valid_operators[exp]['type']=='left_bracket':
          operator_stack.append(exp)
          continue

      if exp in self.valid_operators:
        if self.valid_operators[exp]['type'] == 'right_bracket':
          while True:
            if len(operator_stack)==0:
              raise ValueError('Parenthesis mismatch!')
            if self.valid_operators[operator_stack[-1]]['type']=='left_bracket':
              operator_stack.pop()
              break
            output_stack.append(operator_stack.pop())
          continue

      if exp in self.valid_operators:
        prec=self.valid_operators[exp]['precedence']
        while len(operator_stack)>0:
          if self.valid_operators[operator_stack[-1]]['type']=='left_bracket':
            break
          elif prec>=self.valid_operators[operator_stack[-1]]['precedence']:
            output_stack.append(operator_stack.pop())
          else:
            break
        operator_stack.append(exp)
        continue
      output_stack.append(exp)

    return output_stack

  def _ParseSubExpression(self, subexpression):

    valid_types={'float':'numeric','int':'numeric','string':'string','bool':'bool'}

    column_names=list()
    column_types=list()
   
    final_expression=list()


    for item in subexpression:
      if item in self.col_names:
        column_names.append(item)
        column_types.append(valid_types[self.col_types[self._GetIndex(item)]])
      
    unique_type=list(set(column_types))
    if len(unique_type)>1:
      raise ValueError('Try to compare columns '+','.join(column_names)+' which have inconsistent types!')
    if len(unique_type)==0:
      raise ValueError('Try to evaluate subexpression '+' '.join(subexpression)+' that contains no valid column name of current table!')
    
    for item in subexpression:
      if item in self.valid_operators:
        final_expression.append(item)
        continue
      if item in column_names:
        final_expression.append(item)
        continue
      if unique_type[0]=='numeric':
        if item in ['NaN','nan','None','none']:
          final_expression.append(None)
          continue
        else:
          try:
            final_expression.append(float(item))
            continue
          except:
            raise RuntimeError('Tried to cast '+item+' into numeric type to compare with column(s) '+','.join(column_names)+', but failed!')
      elif unique_type[0]=='bool':
        if item in ['None','none']:
          final_expression.append(None)
          continue
        if item in ['true','True']:
          final_expression.append(True)
          continue
        if item in ['false','False']:
          final_expression.append(False)
          continue
        raise RuntimeError('Tried to cast '+item+' into boolean type to compare with column(s) '+','.join(column_names)+', but failed!')
      elif unique_type[0]=='string':
        final_expression.append(item)

    return final_expression


  def _ParseExpression(self, split_expression):

    #check for problematic cases like 'a<=b<=c'. We don't know which operator to evaluate first
    for i in range(len(split_expression)-3):
      if (split_expression[i] in self.valid_operators) and (split_expression[i+2] in self.valid_operators):
        if self.valid_operators[split_expression[i]]['precedence']==self.valid_operators[split_expression[i+2]]['precedence']:
          raise ValueError('Cannot Evaluate '+' '.join(split_expression[i:i+3])+' since both operators have same precedence!')

    #handle , operator
    #replaces an expression like 'rnum=1,2,3' with '(rnum=1 or rnum=2 or rnum=3)'

    temp_split_expression=list()
    skips=0

    for i in range(len(split_expression)):
      if skips>0:
        skips-=1
        continue
      if ',' in split_expression[i]:

        if split_expression[max(0,i-1)] != '=' and split_expression[min(i+1,len(split_expression)-1)] != '=':
          raise ValueError('Can evaluate \',\' operator only in combination with \"=\" in subexpression ',' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))]))

        single_operands=split_expression[i].split(',')

        if split_expression[max(0,i-1)]=='=':
          if i-2<0:
            raise ValueError('Cannot evaluate subexpression '+' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))])+' starting with an \'=\'')
          main_operand=split_expression[i-2]
          temp_split_expression.pop()
          temp_split_expression.pop()
          skips=0

        else:
          if i+2>len(split_expression)-1:
            raise ValueError('Cannot evaluate subexpression '+' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))])+' ending with an \'=\'')
          main_operand=split_expression[i+2]
          skips=2

        temp_expression=list(['('])
        temp_expression+=' or '.join(['%s = %s'% (a,b) for (a,b) in zip(len(single_operands)*[main_operand],single_operands)]).split()
        temp_expression.append(')')
        temp_split_expression+=temp_expression
        continue

      temp_split_expression.append(split_expression[i])

    split_expression=temp_split_expression

    #handle ':' operator
    #replaces an expression like 'col_a=x:y' with '(col_a>=x and col_a<=y)'
        
    temp_split_expression=list()
    skips=0

    for i in range(len(split_expression)):
      if skips>0:
        skips-=1
        continue
      if ':' in split_expression[i]:
        if split_expression[max(0,i-1)] != '=' and split_expression[min(i+1,len(split_expression)-1)] != '=':
          raise ValueError('Can evaluate subexpression '+' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))])+' \':\' sign is only allowed in combination with \'=\'')
        if len(split_expression[i].split(':')) != 2:
          raise ValueError('Can operate \':\' operator only on 2 operands in subexpression '+' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))]))

        lhs=split_expression[i].split(':')[0]
        rhs=split_expression[i].split(':')[1]

        template_expression=['(','','<=','','and','','<=','',')']

        if split_expression[max(0,i-1)] == '=':
          if i-2<0:
            raise ValueError('Cannot evaluate subexpression '+' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))])+' starting with an \'=\'')
          temp_split_expression.pop()
          temp_split_expression.pop()
          template_expression[3]=split_expression[i-2]
          template_expression[5]=split_expression[i-2]
          skips=0

        else:
          if i+2>len(split_expression)-1:
            raise ValueError('Cannot evaluate subexpression '+' '.join(split_expression[max(0,i-1):min(i+1,len(split_expression))])+' ending with an \'=\'')
          template_expression[3]=split_expression[i+2]
          template_expression[5]=split_expression[i+2]
          skips=2 

        template_expression[1]=lhs
        template_expression[7]=rhs
        temp_split_expression+=template_expression
        continue

      temp_split_expression.append(split_expression[i])
  
    split_expression=temp_split_expression

    #the whole thing is now split to pieces, we need to cast the types of the operands
    #into the types of the columns, the operands are compared against.

    final_expression=list()
    subexpression=list()

    for item in split_expression:
      if item in self.valid_operators:
        if self.valid_operators[item]['type'] in ['boolean','left_bracket','right_bracket']:
          if len(subexpression)>0:
            #figure out the column type(s) and parse the operands
            final_expression+=self._ParseSubExpression(subexpression)
            subexpression=list()
          final_expression.append(item)
          continue
      subexpression.append(item)

    if len(subexpression)>0:
      final_expression+=self._ParseSubExpression(subexpression)

    return final_expression       
      

  def _ExpressionLexer(self, expression):

    #Reads token after token and searches for brackets and valid_operators
    #everything, that doesn't match the above is assumed to be an operand

    split_expression=list()

    actual_position=0
    eaten_stuff=''

    while True:

      if actual_position>=len(expression):
        if len(eaten_stuff)>0:
          split_expression.append(eaten_stuff)
        return split_expression

      token=expression[actual_position]

      if token.isspace():
        if len(eaten_stuff)>0:
          split_expression.append(eaten_stuff)
          eaten_stuff=''
        actual_position+=1
        continue

      #note, that there is no check for boolean operators. They need to be clearly separated by spaces
      #or brackets anyway, so they get added with the eaten stuff
      if token in self.valid_operators:
        if self.valid_operators[token]['type']=='left_bracket' or self.valid_operators[token]['type']=='right_bracket':
          if len(eaten_stuff)>0:
            split_expression.append(eaten_stuff)
            eaten_stuff=''
          split_expression.append(token)
          actual_position+=1
          continue

        if self.valid_operators[token]['type']=='arithmetic':
          if len(eaten_stuff)>0:
            split_expression.append(eaten_stuff)
            eaten_stuff=''
          split_expression.append(token)
          actual_position+=1
          continue

        if self.valid_operators[token]['type']=='comparison':
          if len(eaten_stuff)>0:
            split_expression.append(eaten_stuff)
            eaten_stuff=''
          if self.valid_operators[token]['extension']!=None:
            if actual_position+len(self.valid_operators[token]['extension'])<len(expression):
              if expression[actual_position:actual_position+len(self.valid_operators[token]['extension'])]==self.valid_operators[token]['extension']:
                split_expression.append(self.valid_operators[token]['extension'])
                actual_position+=len(self.valid_operators[token]['extension'])
                continue
          split_expression.append(token)
          actual_position+=1
          continue

      eaten_stuff+=token
      actual_position+=1



import sys
import math
import ost.img.alg

if len(sys.argv)==2:
  image1=io.LoadImage(sys.argv[1])
  image2=io.LoadImage(sys.argv[2])
else:
  image1=io.LoadImage('square.png')
  image2=io.LoadImage('circle.png')
if image1.GetExtent() != image2.GetExtent():
  raise RuntimeError('The input images should have the same size.')
image1.CenterSpatialOrigin()
image2.CenterSpatialOrigin()
image1.ApplyIP(ost.img.alg.DFT())
image2.ApplyIP(ost.img.alg.DFT())
ex_it=img.ExtentIterator(image1.GetExtent())
diff_image=img.CreateImage(image1.GetExtent())
for pixel in ex_it:
  phase1=img.Phase(image1.GetComplex(pixel))
  phase2=img.Phase(image2.GetComplex(pixel))
  phase_diff=phase1-phase2
  diff_image.SetReal(pixel,180.0*float(phase_diff)/math.pi)
v=gui.CreateDataViewer(diff_image,"Phase difference (in degrees)")



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

import sys
from ost import gui
from ost import gfx
try: 
  from ost import img
  from .wireframe_widget import WireframeWidget
  _img_present=True
except ImportError:
  _img_present=False
  pass
from PyQt5 import QtCore, QtWidgets
from .scene_selection_helper import SelHelper
from .combo_options_widget import ComboOptionsWidget
from .custom_widget import CustomWidget
from .cpk_widget import CPKWidget
from .tube_widget import TubeWidget
from .simple_widget import SimpleWidget
from .sline_widget import SlineWidget
from .hsc_widget import HSCWidget
from .trace_widget import TraceWidget
from .line_trace_widget import LineTraceWidget

class RenderOptionsWidget(ComboOptionsWidget):
  def __init__(self, parent=None):
    ComboOptionsWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Render Options"
    
    self.keep_action_ = QtWidgets.QAction("K",self)
    self.keep_action_.setCheckable(True);
    self.keep_action_.setChecked(False)
    self.keep_action_.setToolTip("Keep rendermodes and add current")
    self.keep_button_ = QtWidgets.QToolButton(self)
    self.keep_button_.setDefaultAction(self.keep_action_)
    
    self.grid_layout_.addWidget(self.keep_button_, 0, 1, 1, 1)
    self.grid_layout_.addWidget(self.stacked_widget_, 1, 0, 1, 2)
    
    #Add options to menu    
    self.entity_widgets_ = list()
    self.entity_widgets_.append(["", EmptyMode()])
    self.entity_widgets_.append([gfx.RenderMode.SIMPLE, SimpleWidget()])
    self.entity_widgets_.append([gfx.RenderMode.CUSTOM, CustomWidget()])
    self.entity_widgets_.append([gfx.RenderMode.CPK, CPKWidget()])
    self.entity_widgets_.append([gfx.RenderMode.LINE_TRACE, LineTraceWidget()])
    self.entity_widgets_.append([gfx.RenderMode.TRACE, TraceWidget()])
    self.entity_widgets_.append([gfx.RenderMode.SLINE, SlineWidget()])
    self.entity_widgets_.append([gfx.RenderMode.TUBE, TubeWidget()])
    self.entity_widgets_.append([gfx.RenderMode.HSC, HSCWidget()])

    self.img_widgets_ = list()
    if _img_present:
      self.img_widgets_.append(["", EmptyMode()])
      self.img_widgets_.append([gfx.RenderMode.SIMPLE, WireframeWidget()])
      self.img_widgets_.append([gfx.RenderMode.FILL, EmptyMode("Fill",gfx.RenderMode.FILL)])

    self._in_view_method = False
    self.setMinimumSize(250,200)
  def OnActivate(self, item):
    self.DoResize()
  def OnComboChange(self, item):
    scene_selection = gui.SceneSelection.Instance()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      if isinstance(node, gfx.Entity) or (_img_present and isinstance(node, gfx.MapIso)):
        render_mode = item.GetRenderMode()
        if render_mode is not None:
          node.SetRenderMode(render_mode)
    if(scene_selection.GetActiveViewCount() > 0):
      entity = scene_selection.GetViewEntity()
      view = scene_selection.GetViewUnion()        
      render_mode = item.GetRenderMode()
      if render_mode is not None:
        entity.SetRenderMode(item.GetRenderMode(),view,self.keep_action_.isChecked())
        
    item.Update()
    self.DoResize()
  
  def Update(self):
    if hasattr(self, "keep_button_"):
      self.keep_button_.setEnabled(True)
    
    ComboOptionsWidget.setEnabled(self,True)
    
    cur_widget = self.GetCurrentWidget()
    new_render_mode = None
    if cur_widget is not None:
      new_render_mode = cur_widget.GetRenderMode()
    
    if SelHelper().CheckAllFlags(SelHelper.NO_SELECTION):
      ComboOptionsWidget.setEnabled(self,False)
      return
    
    if not self._in_view_method:
      for w in self.entity_widgets_:
        self.RemoveWidget(w[0])
      for w in self.img_widgets_:
        self.RemoveWidget(w[0])

    scene_selection = gui.SceneSelection.Instance()        
    if scene_selection.GetActiveNodeCount() > 0 :
      if hasattr(self, "keep_button_"):
        self.keep_button_.setEnabled(False)
      render_mode_valid = True
      render_mode = None
      for i in range(0,scene_selection.GetActiveNodeCount()):
        node = scene_selection.GetActiveNode(i)
        if isinstance(scene_selection.GetActiveNode(i), gfx.GfxObj):
          if render_mode is None:
            render_mode = node.GetRenderMode()
          elif render_mode != node.GetRenderMode():
            render_mode_valid = False
 
    if SelHelper().CheckFlags(SelHelper.HAS_IMG | SelHelper.IS_ONE_TYPE):
      for w in self.img_widgets_:
        self.AddWidget(w[0], w[1])
    elif SelHelper().CheckMinOneFlag(SelHelper.HAS_ENTITY| SelHelper.HAS_VIEW) and SelHelper().CheckNotFlags(SelHelper.HAS_IMG):
      if not self._in_view_method:
        for w in self.entity_widgets_:
          self.AddWidget(w[0], w[1])
    else:
      ComboOptionsWidget.setEnabled(self,False)
      return
    
    if SelHelper().CheckMinOneFlag(SelHelper.HAS_ENTITY| SelHelper.HAS_IMG) and SelHelper().CheckNotFlags(SelHelper.HAS_VIEW):
      if(render_mode_valid):
        ComboOptionsWidget.ChangeSelectedItem(self,render_mode)
      else:
        ComboOptionsWidget.ChangeSelectedItem(self,"")
    else:
      if not self._in_view_method:
        self._in_view_method = True
      ComboOptionsWidget.ChangeSelectedItem(self,new_render_mode)
    self.GetCurrentWidget().Update()
      
  def GetText(self):
    return self.text_
        
class EmptyMode(QtWidgets.QWidget):
  def __init__(self, text="", render_mode=None, parent=None):
    QtWidgets.QLabel.__init__(self, parent)
    self.setMinimumSize(250,30)
    self.text = text
    self.render_mode = render_mode
    if(render_mode):
      text_label = QtWidgets.QLabel(text)
      font = text_label.font()
      font.setBold(True)
      grid = QtWidgets.QGridLayout()
      grid.addWidget(text_label,0,0,1,1)
      grid.addWidget(QtWidgets.QLabel("No Settings available"), 1, 0, 1, 3)
      grid.setRowStretch(2,1)
      self.setLayout(grid)
      self.setMinimumSize(250,60)
      
  def Update(self):
   True #Do Nothing

  def GetText(self):
    return self.text

  def GetRenderMode(self):
    return self.render_mode



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
from .render_mode_widget import RenderModeWidget

#Tube Render Options
class TubeWidget(RenderModeWidget):
  def __init__(self, parent=None):
    RenderModeWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Smooth Tube"
    
    #Set Render Mode
    self.mode_ = gfx.RenderMode.TUBE
        
    #Defaults
    min_spline_detail = 1
    max_spline_detail = 20
    
    min_arc_detail = 1
    max_arc_detail = 20
    
    min_radius = 0.1
    max_radius = 2

    min_ratio = 0.2
    max_ratio = 4
    
    #UI
    tube_label = QtWidgets.QLabel("Tube Settings")
    font = tube_label.font()
    font.setBold(True)
    
    #Poly Mode
    poly_mode_label = QtWidgets.QLabel("Poly Mode")
    
    self.poly_mode_cb_ = QtWidgets.QComboBox()
    self.poly_mode_cb_.addItem("Points")
    self.poly_mode_cb_.addItem("Wireframe")
    self.poly_mode_cb_.addItem("Surface")
    
    #Sphere Label
    spline_label = QtWidgets.QLabel("Spline Detail")
    
    self.spline_spinbox_ = QtWidgets.QSpinBox()
    self.spline_spinbox_.setRange(min_spline_detail, max_spline_detail)
    
    #Arc Label
    arc_label = QtWidgets.QLabel("Arc Detail")
    
    self.arc_spinbox_ = QtWidgets.QSpinBox()
    self.arc_spinbox_.setRange(min_arc_detail, max_arc_detail)
        
    #Radius
    radius_label = QtWidgets.QLabel("Tube radius")

    
    self.radius_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.radius_spinbox_.setRange(min_radius, max_radius)
    self.radius_spinbox_.setDecimals(1)
    self.radius_spinbox_.setSingleStep(0.1)

    self.radius_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.radius_slider_.setRange(min_radius*10.0, max_radius*10.0)
    self.radius_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.radius_slider_.setTickInterval(1)
    
    #Ratio
    ratio_label = QtWidgets.QLabel("Tube ratio")
    
    self.ratio_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.ratio_spinbox_.setRange(min_ratio, max_ratio)
    self.ratio_spinbox_.setDecimals(1)
    self.ratio_spinbox_.setSingleStep(0.1)
    self.ratio_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.ratio_slider_.setRange(min_ratio*10.0, max_ratio*10.0)
    self.ratio_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.ratio_slider_.setTickInterval(2)
    
    grid = QtWidgets.QGridLayout()
    grid.addWidget(tube_label,0,0,1,3)
    grid.addWidget(poly_mode_label,1,0,1,3)
    grid.addWidget(self.poly_mode_cb_,1,3,1,2)
    grid.addWidget(spline_label, 2, 0, 1, 3)
    grid.addWidget(self.spline_spinbox_, 2, 4, 1, 1)
    grid.addWidget(arc_label,3,0,1,3)
    grid.addWidget(self.arc_spinbox_,3,4,1,1)
    grid.addWidget(radius_label,4,0,1,1)
    grid.addWidget(self.radius_slider_,4,1,1,3)
    grid.addWidget(self.radius_spinbox_,4,4,1,1)
    grid.addWidget(ratio_label,5,0,1,1)
    grid.addWidget(self.ratio_slider_,5,1,1,3)
    grid.addWidget(self.ratio_spinbox_,5,4,1,1)
    grid.setRowStretch(6,1)
    self.setLayout(grid)
    
    self.spline_spinbox_.valueChanged.connect(self.UpdateSplineDetail)
    self.arc_spinbox_.valueChanged.connect(self.UpdateArcDetail)
    self.poly_mode_cb_.currentIndexChanged.connect(self.UpdatePolyMode)
    self.radius_spinbox_.valueChanged.connect(self.UpdateRadius)
    self.radius_slider_.valueChanged.connect(self.UpdateSliderRadius)
    self.ratio_spinbox_.valueChanged.connect(self.UpdateRatio)
    self.ratio_slider_.valueChanged.connect(self.UpdateSliderRatio)
  
    self.setMinimumSize(250,200)
   
  def UpdateGui(self,options):
    self.poly_mode_cb_.setCurrentIndex(options.GetPolyMode())
    self.spline_spinbox_.setValue(options.GetSplineDetail())
    self.arc_spinbox_.setValue(options.GetArcDetail())
    self.UpdateRadiusGui(options.GetTubeRadius())
    self.UpdateRatioGui(options.GetTubeRatio())
   
  def UpdatePolyMode(self, value):
    self.GetOptions().SetPolyMode(value)
    self.ApplyOptions()
    
  def UpdateSplineDetail(self, value):
    self.GetOptions().SetSplineDetail(value)
    self.ApplyOptions()
        
  def UpdateArcDetail(self, value):
    self.GetOptions().SetArcDetail(value)
    self.ApplyOptions()
    
  def UpdateRadius(self, value):
    self.GetOptions().SetTubeRadius(value)
    self.ApplyOptions()
    
  def UpdateSliderRadius(self, value):
    self.GetOptions().SetTubeRadius(value/10.0)

  def UpdateRatio(self, value):
    self.GetOptions().SetTubeRatio(value)
    
  def UpdateSliderRatio(self, value):
    self.GetOptions().SetTubeRatio(value/10.0)

  def UpdateRadiusGui(self,value):
    value = round(value, 2)
    if(abs(value*10.0 - self.radius_slider_.value())>=self.radius_slider_.singleStep()):
      self.radius_slider_.setValue(value*10.0)
    if(abs(value - self.radius_spinbox_.value())>=self.radius_spinbox_.singleStep()):
      self.radius_spinbox_.setValue(value)
  
  def UpdateRatioGui(self,value):
    value = round(value, 2)
    if(abs(value*10.0 - self.ratio_slider_.value())>=self.ratio_slider_.singleStep()):
      self.ratio_slider_.setValue(value*10.0)
    if(abs(value - self.ratio_spinbox_.value())>=self.ratio_spinbox_.singleStep()):
      self.ratio_spinbox_.setValue(value)

  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



INFO_PLIST="""<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" 
    "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
  <key>CFBundleDevelopmentRegion</key>
  <string>English</string>
  <key>CFBundleExecutable</key>
  <string>dng</string>
  <key>CFBundleName</key>
  <string>DNG</string>  
  <key>CFBundleDisplayName</key>
  <string>DNG</string>  
  <key>CFBundleHelpBookFolder</key>
  <string>OpenStructure Manual</string>
  <key>CFBundleHelpBookName</key>
  <string>OpenStructure Help</string>
  <key>CFBundleIconFile</key>
  <string>icon.icns</string>
  <key>CFBundleIdentifier</key>
  <string>org.openstructure.OpenStructure</string>
  <key>CFBundleInfoDictionaryVersion</key>
  <string>6.0</string>
  <key>CFBundlePackageType</key>
  <string>APPL</string>
  <key>CFBundleSignature</key>
  <string>????</string>
  <key>CFBundleVersion</key>
  <string></string>
  <key>NSMainNibFile</key>
  <string>MainMenu</string>
  <key>NSPrincipalClass</key>
  <string>NSApplication</string>
</dict>
</plist>"""

import os
from shutil import copy
import stat
def _WritePkgInfo(bundle):
    pkg_info=open(os.path.join(bundle+'.app', 'Contents/PkgInfo'), 'w+')
    pkg_info.write('APPL????')
    pkg_info.close()

def _WriteInfoPList(bundle):
    info_plist=open(os.path.join(bundle+'.app', 'Contents/Info.plist'), 'w+')
    info_plist.write(INFO_PLIST)
    info_plist.close()

def _WriteScript(bundle, standalone):
  if standalone:
    script="""#!/bin/sh
if [ -h "$0" ] ; then
  SCRIPT_NAME=`readlink "$0"`
else
  SCRIPT_NAME="$0"
fi
`dirname "$SCRIPT_NAME"`/../bin/dng
    """
  else:
    script="""#!/bin/sh
if [ -e $HOME/Library/OpenStructure/bin/dng ]; then
  $HOME/Library/OpenStructure/bin/dng
else
  /Library/OpenStructure/bin/dng
fi
    """
  bin_path=os.path.join('%s.app' % bundle, 'Contents/MacOS/dng')
  bin=open(bin_path, 'w+')
  bin.write(script)
  bin.close()
  os.system('chmod a+x %s' % bin_path)

def _CreateBundleSkeleton(bundle, standalone):
  bin_path=os.path.join('%s.app' % bundle, 'Contents/MacOS')
  if not os.path.exists(bin_path):
    os.makedirs(bin_path)
  _WritePkgInfo(bundle)
  _WriteInfoPList(bundle)
  _WriteScript(bundle, standalone)
  res_dir='%s.app/Contents/Resources' % bundle
  if not os.path.exists(res_dir):
    os.makedirs(res_dir)  
  copy('../../graphics/icon.icns', res_dir)

def create_bundle(bundle_name, standalone=False):
  _CreateBundleSkeleton(bundle_name, standalone)
  if standalone:
    os.system('mv standalone/* DNG.app/Contents/')


import desost_util as util

e=io.LoadPDB("../../../examples/demos/data/sdh.pdb")

s=util.CreateSurf(e.Select("chain=A"),density=4)

scene.Add(gfx.Entity("trace",gfx.TUBE,e.Select("chain=A")))
scene["trace"].tube_options.arc_detail=8
scene["trace"].tube_options.spline_detail=8
scene["trace"].tube_options.tube_radius=1.0
grad=gfx.Gradient()
grad.SetColorAt(0.0,gfx.RED)
grad.SetColorAt(0.5,gfx.GREEN)
grad.SetColorAt(1.0,gfx.BLUE)
scene["trace"].ColorBy("rnum",grad)

scene.Add(gfx.Surface("surf",s))
scene["surf"].ColorBy("rnum",grad)

if not gui_mode:
  ge = gfx.GostExporter("test.gost")
  scene.Export(ge)



import sys
import unittest
from ost import *
from ost import settings
from ost.bindings import blast
import re
import tempfile
import os
import shutil

class TestBlastBindings(unittest.TestCase):
  def setUp(self):
    self.query=seq.CreateSequence('A', 'MKPHPWFFGKIPRAKAEEMLSKQRHDGAFLIRESESAPG'+
                                       'DFSLSVKFGNDVQHFKVLRDGAGKYFLWVVKFNSLNELV'+
                                       'DYHRSTSVSRNQQIFLRDIEQVP')
  def testAllowedMatricesAndPenalties(self):
    blast.Blast(self.query, 'testfiles/seqdb', matrix='BLOSUM45', 
                gap_open=13, gap_ext=3)
    blast.Blast(self.query, 'testfiles/seqdb', matrix='BLOSUM80')
    blast.Blast(self.query, 'testfiles/seqdb', matrix='PAM70')
    blast.Blast(self.query, 'testfiles/seqdb', matrix='PAM30', gap_open=7,
                gap_ext=2)
    blast.Blast(self.query, 'testfiles/seqdb', matrix='BLOSUM62')
    self.assertRaises(ValueError, blast.Blast, self.query, 'testfiles/seqdb', 
                      matrix='MUAHA')
    self.assertRaises(RuntimeError, blast.Blast, self.query, 'testfiles/seqdb',
                      blast_location='/I/do/not/exist')

  def testMissingDB(self):
    self.assertRaises(IOError, blast.Blast, self.query, 
                      'testfiles/nonexistentdb')
  
  def testBlastExec(self):

    hits = blast.Blast(self.query, 'testfiles/seqdb')

    expected_perfect_hit_ids = ["1fhsA", "3n84E", "1griA"]
    perfect_hit_ids = []
    
    for h in hits:
      perfect_match = False
      for aln_patch in h.aligned_patches:
        if aln_patch.seqid == 1.0:
          perfect_match = True
      if perfect_match:
        perfect_hit_ids.append(h.identifier)

    expected_perfect_hit_ids.sort()    
    perfect_hit_ids.sort()
    
    self.assertEqual(perfect_hit_ids, expected_perfect_hit_ids)

  def testBlastParseOutput(self):

    with open('testfiles/raw_blastout.txt','r') as f:
      raw_out=f.read()

    parsed_out=blast.ParseBlastOutput(raw_out)

    # numbers and ids can be specific to a certain blast version
    expected_ids=['1griA','1fhsA','3n84E','1mw9X']
    expected_evalues=[4.80893e-59,4.80893e-59,9.06925e-58,2.96523]
    expected_scores=[534,534,523,27]
    expected_bitscores=[210.305,210.305,206.068,15.0086]
    expected_seqid=[1.0,1.0,1.0,0.30769]

    for i in range(4):
      patch=parsed_out[i].aligned_patches[0]
      self.assertEqual(expected_ids[i],parsed_out[i].identifier)
      self.assertAlmostEqual(patch.evalue,expected_evalues[i])
      self.assertAlmostEqual(patch.score,expected_scores[i])
      self.assertAlmostEqual(patch.bit_score,expected_bitscores[i])
      self.assertAlmostEqual(patch.seqid,expected_seqid[i],places=5)

  def testBlastVersion(self):

    self.assertRaises(RuntimeError, blast.Blast, self.query, 'testfiles/seqdb',
                      blast_location='/I/do/not/exist')
    version=blast.BlastVersion()
    self.assertTrue(isinstance(version,str))
    re_v = re.compile('\d+\.\d+\.\d+')
    match=False
    if re_v.match(version):
      match=True
    self.assertEqual(match,True)

  def testBuildDatabase(self):

    tmp_dir_name=tempfile.mkdtemp()
    db_name=os.path.join(tmp_dir_name,'testdb')
    blast.CreateDB('testfiles/multiple.fasta',db_name)
    self.assertRaises(IOError,blast.CreateDB, 'testfiles/multiple.fasta',db_name,
                      mkdb_cmd='I/am/not/a/damn/executable')

    test_seq=seq.CreateSequence('B','ALRLVKDGFAVAIADYNDATATAVAAEINQAGGRAVAIKVDV'+
                                    'SRRDQVFAAVEQARKALGGFNVIVNNAGIAPSTPIESIT')
    blastout=blast.Blast(test_seq,db_name)
    self.assertTrue(len(blastout)>10)
    shutil.rmtree(tmp_dir_name)

if __name__ == "__main__":
  # test if blast package is available on system, otherwise ignore tests
  try:
    blastpath=settings.Locate(('blastp','blastall'))
  except(settings.FileNotFound):
    print("Could not find blast executable: ignoring unit tests")
    sys.exit(0)
  from ost import testutils
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import info
from PyQt5 import QtCore, QtWidgets, QtGui

#Gradient Editor
class ImmutableGradientInfoHandler:
  def __init__(self, fileName):
    
    self.FILE_NAME = fileName    
    self.GRADIENTS_GROUP_NAME = "Gradients"
    self.GRADIENT_GROUP_NAME = "Gradient"
    self.NAME_ATTRIBUTE_NAME = "Name"
    
    self.handle_ = info.LoadOrCreateInfo(self.FILE_NAME)
        
    self.root_ = self.handle_.Root()
    
    self.gradients_ = self.root_.RetrieveGroup(self.GRADIENTS_GROUP_NAME)

    
  def GetGfxGradient(self, name):
    group = self.gradients_.GetGroup(name)
    gfx_gradient = gfx.Gradient.GradientFromInfo(group)
    return gfx_gradient
  
  def GetQGradient(self, name):
    gfx_gradient = self.GetGfxGradient(name)
    return ImmutableGradientInfoHandler.ConvertToQGradient(gfx_gradient)
    
  def GetQGradients(self):
    group_list = self.gradients_.GetGroups(self.GRADIENT_GROUP_NAME)
    q_gradients = dict()
    
    for group in group_list:
      if group.HasAttribute(self.NAME_ATTRIBUTE_NAME):
        gfx_gradient = gfx.Gradient.GradientFromInfo(group)
        q_gradients[group.GetAttribute(self.NAME_ATTRIBUTE_NAME)] = (ImmutableGradientInfoHandler.ConvertToQGradient(gfx_gradient))
      
    return q_gradients

  @staticmethod
  def ConvertToQGradient(gfx_gradient):
    stops = gfx_gradient.GetStops()
    qgrad = QtGui.QLinearGradient()
    for stop in stops:
      color = stop.GetColor()
      qcolor = QtGui.QColor(color.Red()*255,
                            color.Green()*255,
                            color.Blue()*255,
                            color.Alpha()*255)
      qgrad.setColorAt(stop.GetRel(), qcolor) 
    return qgrad 
  
  @staticmethod
  def ConvertToGfxGradient(gradient):  
    gfx_gradient = gfx.Gradient()    
    for s in gradient.stops():
      rel=s[0]
      color=s[1]
      gfx_gradient.SetColorAt(s[0], gfx.RGB(s[1].redF(), s[1].greenF(), s[1].blueF()));
    return gfx_gradient;



import unittest
from ost import *
from ost import settings
from ost.bindings import msms

class TestMSMSBindings(unittest.TestCase):
  
  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb")

    self.num_tri=111546
    self.num_vert=55762
    self.ases=20051.928
    self.asas=21574.324
    self.volume=60229.848

  def testCalculateSurface(self):
    surf=msms.CalculateSurface(self.protein,
                               msms_env='MSMSSERVER',
                               density=3,
                               radius=1.4,
                               attach_asa="asa",
                               attach_esa="esa")[0]
    self.assertEqual(self.num_vert, len(surf.GetVertexIDList()))
    self.assertEqual(self.num_tri, len(surf.GetTriIDList()))
 
  def testCalculateSurfaceArea(self):
    (msms_ases, msms_asas)=msms.CalculateSurfaceArea(self.protein, \
                                                     msms_env='MSMSSERVER',
                                                     density=3,
                                                     radius=1.4)
    self.assertAlmostEqual(self.ases, msms_ases[0], 1)
    self.assertAlmostEqual(self.asas, msms_asas[0], 1)

  def testCalculateSurfaceVolume(self):
    volume=msms.CalculateSurfaceVolume(self.protein, \
                                       msms_env='MSMSSERVER',
                                       density=3,
                                       radius=1.4)
    self.assertAlmostEqual(self.volume, volume, 1)

if __name__ == "__main__":
  # test if msms package is available on system, otherwise ignore tests
  VERSION_REQUIRED = '2.6.1'
  try:
    msms._GetExecutable(msms_exe=None, msms_env='MSMSSERVER')
  except(settings.FileNotFound):
    print("Could not find msms executable: ignoring unit tests")
    exit(0)
  version = msms.GetVersion(msms_exe=None, msms_env='MSMSSERVER')
  if version!=VERSION_REQUIRED:
    print("MSMS version (%s) does not match required version %s: ignoring unit tests"%(version, VERSION_REQUIRED))
  from ost import testutils
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

import sys
from ost import gui
from ost import gfx

class SceneObserverImpl(gfx.SceneObserver):
  def __init__(self):
    gfx.SceneObserver.__init__(self)
    self.obs_ = list()                              
  
  def AttachObserver(self, ob):
    self.obs_.append(ob)
  
  def DettachObserver(self, ob):
    self.obs_.remove(ob)
  
  def NodeAdded(self, node):
    for ob in self.obs_ :
      if hasattr(ob, "NodeAdded"):
        ob.NodeAdded(node)

  def NodeRemoved(self, node):
    for ob in self.obs_ :
      if hasattr(ob, "NodeRemoved"):
        ob.NodeRemoved(node)
  
  def NodeChanged(self, node):
    for ob in self.obs_ :
      if hasattr(ob, "NodeChanged"):
        ob.NodeChanged(node)
    
  def SelectionModeChanged(self, sel_mode):
    for ob in self.obs_ :
      if hasattr(ob, "SelectionModeChanged"):
        ob.SelectionModeChanged(node)    
  
  def SelectionChanged(self, obj, sel):
    for ob in self.obs_ :
      if hasattr(ob, "SelectionChanged"):
        ob.SelectionChanged(node)    

  def NodeTransformed(self, node):
      for ob in self.obs_ :
        if hasattr(ob, "NodeTransformed"):
          ob.NodeTransformed(node) 
    
  def NodeRemovedDefault(self, node):
    for ob in self.obs_ :
      if hasattr(ob, "NodeRemovedDefault"):
        ob.NodeRemovedDefault(node)
    
  def RenderModeChanged(self, node):
    for ob in self.obs_ :
      if hasattr(ob, "RenderModeChanged"):
        ob.RenderModeChanged(node)



from ost import io, mol, settings
import unittest
import os
from ost.bindings import naccess
from ost.bindings import dssp


class AccessibilityContainer:
  def __init__(self):
    self.overall_acc = None
    self.per_atom_acc = list()
    self.per_atom_acc_single_chain = list()


def AccessibilitiesRaw(ent, use_naccess = False):

  acc_container = AccessibilityContainer()

  if use_naccess:
    acc_container.overall_acc = naccess.CalculateSurfaceArea(ent)
  else:
    acc_container.overall_acc = mol.alg.Accessibility(ent) 

  for a in ent.atoms:
    acc_container.per_atom_acc.append(a.GetFloatProp("asaAtom"))

  for c in ent.chains:
    chain_selection = ent.Select("cname="+c.GetName())
    if use_naccess:
      naccess.CalculateSurfaceArea(chain_selection)
    else:
      mol.alg.Accessibility(chain_selection)

  for a in ent.atoms:
    acc_container.per_atom_acc_single_chain.append(a.GetFloatProp("asaAtom"))

  return acc_container


def AccessibilitiesOligo(ent):

  acc_container = AccessibilityContainer()

  acc_container.overall_acc = mol.alg.Accessibility(ent, oligo_mode=True)
  
  for a in ent.atoms:
    acc_container.per_atom_acc.append(a.GetFloatProp("asaAtom"))
    acc = a.GetFloatProp("asaAtom_single_chain")
    acc_container.per_atom_acc_single_chain.append(acc)

  return acc_container


def Compare(acc_one, acc_two):

  # 1 digits is sufficient for summed overall accessibility. 
  # In both cases, we should have the full accessibility of the full complex
  if abs(acc_one.overall_acc - acc_two.overall_acc) > 0.1:
    return False

  # the per atom values should be a bit more accurate => 2 digits
  # let's check the per atom accessibility of the full complex
  if len(acc_one.per_atom_acc) != len(acc_two.per_atom_acc):
    return False
  for a,b in zip(acc_one.per_atom_acc, acc_two.per_atom_acc):
    if abs(a - b) > 0.01:
      return False

  # let's check the per atom accessibility when only single chains 
  # are considered
  if len(acc_one.per_atom_acc_single_chain) != \
     len(acc_two.per_atom_acc_single_chain):
    return False
  for a,b in zip(acc_one.per_atom_acc_single_chain, \
                 acc_two.per_atom_acc_single_chain):
    if abs(a - b) > 0.01:
      return False

  return True


class TestAccessibility(unittest.TestCase):
  
  def testAccNACCESS(self):

    # tests oligo mode by comparing the results from doing the
    # corresponding calculations manually
    ent_one = io.LoadPDB(os.path.join("testfiles", "1a0s.pdb"))
    ent_two = io.LoadPDB(os.path.join("testfiles", "1a0s.pdb"))

    # we're only interested in peptide stuff...
    ent_one = ent_one.Select("peptide=true")
    ent_two = ent_two.Select("peptide=true")

    acc_classic = AccessibilitiesRaw(ent_one)
    acc_oligo = AccessibilitiesOligo(ent_two)

    self.assertTrue(Compare(acc_classic, acc_oligo))

    # if there is naccess around, we also check for equality with 
    # naccess results 
    try:
      naccess_path = settings.Locate("naccess")
      ent_three = io.LoadPDB(os.path.join("testfiles", "1a0s.pdb"))
      ent_three = ent_three.Select("peptide=true")
      acc_naccess = AccessibilitiesRaw(ent_three, use_naccess=True)
      self.assertTrue(Compare(acc_classic, acc_naccess))
    except:
      print("Could not find NACCESS, could not compare Accessiblity function...")


  def testAccDSSP(self):

    # only relevant if dssp there
    try:
      # same check used in dssp binding
      dssp_path = settings.Locate(['dsspcmbi', 'dssp', 'mkdssp'],
                                  env_name='DSSP_EXECUTABLE')
    except:
      print("Could not find DSSP, could not compare Accessibility function...")
      return

    # we assume oligo mode to be working as it is tested in 
    # testAccNACCESS. So we only test the single residue
    # accessibilitities
    ent_one = io.LoadPDB(os.path.join("testfiles", "1a0s.pdb"))
    ent_two = io.LoadPDB(os.path.join("testfiles", "1a0s.pdb"))
    ent_one = ent_one.Select("peptide=true")
    ent_two = ent_two.Select("peptide=true")

    dssp.AssignDSSP(ent_one, extract_burial_status=True, dssp_bin=dssp_path)
    mol.alg.Accessibility(ent_two, algorithm=mol.alg.AccessibilityAlgorithm.DSSP)

    for a,b in zip(ent_one.residues, ent_two.residues):

      # overall accessibility
      if a.HasProp("solvent_accessibility") and b.HasProp("asaAbs"):
        diff = abs(a.GetFloatProp("solvent_accessibility") -\
                   round(b.GetFloatProp("asaAbs")))
        self.assertTrue(diff < 0.01)

      # relative accessibility
      if a.HasProp("relative_solvent_accessibility") and b.HasProp("asaRel"):
        diff = abs(a.GetFloatProp("relative_solvent_accessibility") -\
                   b.GetFloatProp("asaRel"))
        self.assertTrue(diff < 0.01)


if __name__ == "__main__":

  from ost import testutils
  testutils.RunTests()



import unittest
import ost

# helper log sink to campture messages
class _CapturingLogSink(ost.LogSink):
  def __init__(self):
    ost.LogSink.__init__(self)
  def LogMessage(self, message, severity):
    self.message = message
    self.severity = severity

# Altough the logging system might appear to be too simple to be worth writing a 
# specific test case for, it actually isn't. The python export is very fragile 
# and seemingly trivial changes can break the code in unexpected ways. So let's 
# check for some invariants
class TestLog(unittest.TestCase):

  def testGetLogSink(self):
    logsink = ost.GetCurrentLogSink()
    self.assertTrue(hasattr(logsink, 'LogMessage'))
    # Check if the return type of logsink is sane
    ost.PushLogSink(ost.GetCurrentLogSink())

  def testPushPopLogSink(self):
    class MyLogSink(ost.LogSink):
       def __init__(self):
         ost.LogSink.__init__(self)
    ls = MyLogSink()
    ost.PushLogSink(ls)
    self.assertEqual(ls, ost.GetCurrentLogSink())
    ost.PopLogSink()
    self.assertNotEqual(ls, ost.GetCurrentLogSink())

  def testLogMessage(self):
    ls = _CapturingLogSink()
    ost.PushVerbosityLevel(1)
    ost.PushLogSink(ls)
    ost.LogError('error message')
    self.assertEqual(ls.message, 'error message\n')
    self.assertEqual(ls.severity, 0)
    ost.LogWarning(1, 2, 3)
    self.assertEqual(ls.message, '1 2 3\n')
    self.assertEqual(ls.severity, 1)
    ost.PopLogSink()

  def testLogMultipleMessages(self):
    # observation: converting non-strings in logging can break following calls
    ls = _CapturingLogSink()
    ost.PushVerbosityLevel(1)
    ost.PushLogSink(ls)
    ost.LogWarning('foo')
    self.assertEqual(ls.message, 'foo\n')
    ost.LogWarning(1)
    self.assertEqual(ls.message, '1\n')
    ost.LogWarning('bar')
    self.assertEqual(ls.message, 'bar\n')
    ost.PopLogSink()

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()


import math,random
from ost import img
mh=img.CreateMap(img.Size(100, 25, 25))
for p in img.ExtentIterator(mh.GetExtent()):
  val=5*math.sin(0.4*math.sqrt(p[0]*p[0]+p[1]*p[1]))+7*math.cos(0.6*math.sqrt(p[2]*p[2]+p[1]*p[1]))
  mh.SetReal(p,val*(1.0+0.0*random.random()))

go=gfx.MapIso("test",mh,4.0)
scene.Add(go)
scene.SetCenter(go.GetCenter())
go.SetLineWidth(1.5)



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
from .render_mode_widget import RenderModeWidget

#Tube Render Options
class HSCWidget(RenderModeWidget):
  def __init__(self, parent=None):
    RenderModeWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Helix & Strand Cartoon"
     
    #Set Render Mode
    self.mode_ = gfx.RenderMode.HSC
    
    #Defaults
    min_spline_detail = 1
    max_spline_detail = 20
    
    min_arc_detail = 1
    max_arc_detail = 20
    
    min_width = 0.1
    max_width = 5.0
    max_tube_width= 5.0

    min_tube_ratio = 0.2
    max_tube_ratio = 4
    min_ratio = 0.1
    max_ratio = 2.5
    
    min_ecc = 0.1
    max_ecc = 5

    min_profile=0
    max_profile=4
    
    #########UI##########
    
    #Poly Mode
    poly_mode_label = QtWidgets.QLabel("Poly Mode")
    
    self.poly_mode_cb_ = QtWidgets.QComboBox()
    self.poly_mode_cb_.addItem("Points")
    self.poly_mode_cb_.addItem("Wireframe")
    self.poly_mode_cb_.addItem("Surface")
    
    #Sphere Label
    spline_label = QtWidgets.QLabel("Spline Detail")
    
    self.spline_spinbox_ = QtWidgets.QSpinBox()
    self.spline_spinbox_.setRange(min_spline_detail, max_spline_detail)
    
    #Arc Label
    arc_label = QtWidgets.QLabel("Arc Detail")
    
    self.arc_spinbox_ = QtWidgets.QSpinBox()
    self.arc_spinbox_.setRange(min_arc_detail, max_arc_detail)
    
    #Tube
    tube_label = QtWidgets.QLabel("Tube")
    font = tube_label.font()
    font.setBold(True)
    
    #Tube Radius
    radius_tube_label = QtWidgets.QLabel("Radius")
    
    self.width_tube_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.width_tube_spinbox_.setRange(min_width, max_tube_width)
    self.width_tube_spinbox_.setDecimals(1)
    self.width_tube_spinbox_.setSingleStep(0.1)
    
    self.width_tube_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.width_tube_slider_.setRange(min_width*10.0, max_tube_width*10.0)
    self.width_tube_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.width_tube_slider_.setTickInterval(1)
    
    #Tube Ratio
    ratio_tube_label = QtWidgets.QLabel("Ratio")
    
    self.thickness_tube_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.thickness_tube_spinbox_.setRange(min_tube_ratio, max_tube_ratio)
    self.thickness_tube_spinbox_.setDecimals(1)
    self.thickness_tube_spinbox_.setSingleStep(0.1)
    
    self.thickness_tube_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.thickness_tube_slider_.setRange(min_tube_ratio*10.0, max_tube_ratio*10)
    self.thickness_tube_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.thickness_tube_slider_.setTickInterval(1)

    # Tube Profile Type
    tube_profile_label = QtWidgets.QLabel("Tube Profile Type")
    self.tube_profile_spinbox_ = QtWidgets.QSpinBox()
    self.tube_profile_spinbox_.setRange(min_profile, max_profile)
    
    # Helix
    helix_label = QtWidgets.QLabel("Helix")
    font = helix_label.font()
    font.setBold(True)
    
    #Helix Radius
    radius_helix_label = QtWidgets.QLabel("Width")
    
    self.width_helix_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.width_helix_spinbox_.setRange(min_width, max_width)
    self.width_helix_spinbox_.setDecimals(1)
    self.width_helix_spinbox_.setSingleStep(0.1)
    
    self.width_helix_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.width_helix_slider_.setRange(min_width*10.0, max_width*10.0)
    self.width_helix_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.width_helix_slider_.setTickInterval(1)
    
    #Helix Ratio
    ratio_helix_label = QtWidgets.QLabel("Thickness")
    
    self.thickness_helix_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.thickness_helix_spinbox_.setRange(min_ratio,max_ratio)
    self.thickness_helix_spinbox_.setDecimals(1)
    self.thickness_helix_spinbox_.setSingleStep(0.1)
    
    self.thickness_helix_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.thickness_helix_slider_.setRange(min_ratio*10.0,max_ratio*10.0)
    self.thickness_helix_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.thickness_helix_slider_.setTickInterval(1)
    
    #Helix ECC
    ecc_helix_label = QtWidgets.QLabel("ECC")
    
    self.ecc_helix_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.ecc_helix_spinbox_.setRange(min_ecc,max_ecc)
    self.ecc_helix_spinbox_.setDecimals(1)
    self.ecc_helix_spinbox_.setSingleStep(0.1)
    
    self.ecc_helix_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.ecc_helix_slider_.setRange(min_ecc*10,max_ecc*10)
    self.ecc_helix_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.ecc_helix_slider_.setTickInterval(1)
    
    # Helix Profile Type
    helix_profile_label = QtWidgets.QLabel("Helix Profile Type")
    self.helix_profile_spinbox_ = QtWidgets.QSpinBox()
    self.helix_profile_spinbox_.setRange(min_profile, max_profile)
    
    # Helix Rendering Mode
    helix_mode_label = QtWidgets.QLabel("Helix Render Mode")
    self.helix_mode_spinbox_ = QtWidgets.QSpinBox()
    self.helix_mode_spinbox_.setRange(0, 1)
    
    #Strand
    strand_label = QtWidgets.QLabel("Strand")
    font = strand_label.font()
    font.setBold(1)
    
    #Strand Radius
    radius_strand_label = QtWidgets.QLabel("Width")
    
    self.width_strand_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.width_strand_spinbox_.setRange(min_width, max_width)
    self.width_strand_spinbox_.setDecimals(1)
    self.width_strand_spinbox_.setSingleStep(0.1)
    
    self.width_strand_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.width_strand_slider_.setRange(min_width*10, max_width*10)
    self.width_strand_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.width_strand_slider_.setTickInterval(1)
    
    #Strand Ratio
    ratio_strand_label = QtWidgets.QLabel("Thickness")
    
    self.thickness_strand_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.thickness_strand_spinbox_.setRange(min_ratio,max_ratio)
    self.thickness_strand_spinbox_.setDecimals(1)
    self.thickness_strand_spinbox_.setSingleStep(0.1)
    
    self.thickness_strand_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.thickness_strand_slider_.setRange(min_ratio*10,max_ratio*10)
    self.thickness_strand_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.thickness_strand_slider_.setTickInterval(1)
    
    #Strand ECC
    ecc_strand_label = QtWidgets.QLabel("ECC")
    
    self.ecc_strand_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.ecc_strand_spinbox_.setRange(min_ecc,max_ecc)
    self.ecc_strand_spinbox_.setDecimals(1)
    self.ecc_strand_spinbox_.setSingleStep(0.1)
    
    self.ecc_strand_slider_ = QtWidgets.QSlider(QtCore.Qt.Horizontal, self)
    self.ecc_strand_slider_.setRange(min_ecc*10,max_ecc*10)
    self.ecc_strand_slider_.setTickPosition(QtWidgets.QSlider.NoTicks)
    self.ecc_strand_slider_.setTickInterval(1)  
    
    # Strand Profile Type
    strand_profile_label = QtWidgets.QLabel("Strand Profile Type")
    self.strand_profile_spinbox_ = QtWidgets.QSpinBox()
    self.strand_profile_spinbox_.setRange(min_profile, max_profile)

    # Strand Rendering Mode
    strand_mode_label = QtWidgets.QLabel("Strand Render Mode")
    self.strand_mode_spinbox_ = QtWidgets.QSpinBox()
    self.strand_mode_spinbox_.setRange(0, 1)

    #
    row=1
    grid = QtWidgets.QGridLayout()
    grid.addWidget(poly_mode_label,row,0,1,1)
    grid.addWidget(self.poly_mode_cb_,row,3,1,2)
    row+=1
    grid.addWidget(spline_label, row, 0, 1, 3)
    grid.addWidget(self.spline_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(arc_label,row,0,1,3)
    grid.addWidget(self.arc_spinbox_,row,4,1,1)
    row+=1
    grid.addWidget(tube_label, row, 0, 1, 3)
    row+=1
    grid.addWidget(radius_tube_label, row, 0, 1, 1)
    grid.addWidget(self.width_tube_slider_, row, 1, 1, 3)
    grid.addWidget(self.width_tube_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(ratio_tube_label, row, 0, 1, 1)
    grid.addWidget(self.thickness_tube_slider_, row, 1, 1, 3)
    grid.addWidget(self.thickness_tube_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(tube_profile_label, row, 0, 1, 3)
    grid.addWidget(self.tube_profile_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(helix_label, row, 0, 1, 3)
    row+=1
    grid.addWidget(radius_helix_label, row, 0, 1, 1)
    grid.addWidget(self.width_helix_slider_, row, 1, 1, 3)
    grid.addWidget(self.width_helix_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(ratio_helix_label, row, 0, 1, 1)
    grid.addWidget(self.thickness_helix_slider_, row, 1, 1, 3)
    grid.addWidget(self.thickness_helix_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(ecc_helix_label, row, 0, 1, 1)
    grid.addWidget(self.ecc_helix_slider_, row, 1, 1, 3)
    grid.addWidget(self.ecc_helix_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(helix_profile_label, row, 0, 1, 3)
    grid.addWidget(self.helix_profile_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(helix_mode_label, row, 0, 1, 3)
    grid.addWidget(self.helix_mode_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(strand_label, row, 0, 1, 3)
    row+=1
    grid.addWidget(radius_strand_label, row, 0, 1, 1)
    grid.addWidget(self.width_strand_slider_, row, 1, 1, 3)
    grid.addWidget(self.width_strand_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(ratio_strand_label, row, 0, 1, 1)
    grid.addWidget(self.thickness_strand_slider_, row, 1, 1, 3)
    grid.addWidget(self.thickness_strand_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(ecc_strand_label, row, 0, 1, 1)
    grid.addWidget(self.ecc_strand_slider_, row, 1, 1, 3)
    grid.addWidget(self.ecc_strand_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(strand_profile_label, row, 0, 1, 3)
    grid.addWidget(self.strand_profile_spinbox_, row, 4, 1, 1)
    row+=1
    grid.addWidget(strand_mode_label, row, 0, 1, 3)
    grid.addWidget(self.strand_mode_spinbox_, row, 4, 1, 1)
    
    grid.setRowStretch(row+1,1)
    self.setLayout(grid)
    
    self.spline_spinbox_.valueChanged.connect(self.UpdateSplineDetail)
    self.arc_spinbox_.valueChanged.connect(self.UpdateArcDetail)
    self.poly_mode_cb_.currentIndexChanged.connect(self.UpdatePolyMode)
    
    self.width_tube_spinbox_.valueChanged.connect(self.UpdateTubeRadius)
    self.width_tube_slider_.valueChanged.connect(self.UpdateSliderTubeRadius)
    self.thickness_tube_spinbox_.valueChanged.connect(self.UpdateTubeRatio)
    self.thickness_tube_slider_.valueChanged.connect(self.UpdateSliderTubeRatio)
    self.tube_profile_spinbox_.valueChanged.connect(self.UpdateTubeProfileType)
    
    self.width_helix_spinbox_.valueChanged.connect(self.UpdateHelixWidth)
    self.width_helix_slider_.valueChanged.connect(self.UpdateSliderHelixWidth)
    self.thickness_helix_spinbox_.valueChanged.connect(self.UpdateHelixThickness)
    self.thickness_helix_slider_.valueChanged.connect(self.UpdateSliderHelixThickness)
    self.ecc_helix_spinbox_.valueChanged.connect(self.UpdateHelixEcc)
    self.ecc_helix_slider_.valueChanged.connect(self.UpdateSliderHelixEcc)
    self.helix_profile_spinbox_.valueChanged.connect(self.UpdateHelixProfileType)
    self.helix_mode_spinbox_.valueChanged.connect(self.UpdateHelixMode)

    self.width_strand_spinbox_.valueChanged.connect(self.UpdateStrandWidth)
    self.width_strand_slider_.valueChanged.connect(self.UpdateSliderStrandWidth)
    self.thickness_strand_spinbox_.valueChanged.connect(self.UpdateStrandThickness)
    self.thickness_strand_slider_.valueChanged.connect(self.UpdateSliderStrandThickness)
    self.ecc_strand_spinbox_.valueChanged.connect(self.UpdateStrandEcc)
    self.ecc_strand_slider_.valueChanged.connect(self.UpdateSliderStrandEcc)    
    self.strand_profile_spinbox_.valueChanged.connect(self.UpdateStrandProfileType)
    self.strand_mode_spinbox_.valueChanged.connect(self.UpdateStrandMode)
    
    self.setMinimumSize(250,550)
    ########/UI########
  
  def UpdateGui(self,options):
    self.poly_mode_cb_.setCurrentIndex(options.GetPolyMode())
    self.spline_spinbox_.setValue(options.GetSplineDetail())
    self.arc_spinbox_.setValue(options.GetArcDetail())
    
    self.UpdateTubeRadiusGui(options.GetTubeRadius())
    self.UpdateTubeRatioGui(options.GetTubeRatio())
    self.tube_profile_spinbox_.setValue(options.GetTubeProfileType())
    self.UpdateHelixWidthGui(options.GetHelixWidth())
    self.UpdateHelixThicknessGui(options.GetHelixThickness())
    self.UpdateHelixEccGui(options.GetHelixEcc())
    self.helix_profile_spinbox_.setValue(options.GetHelixProfileType())
    self.helix_mode_spinbox_.setValue(options.GetHelixMode())
    self.UpdateStrandWidthGui(options.GetStrandWidth())
    self.UpdateStrandThicknessGui(options.GetStrandThickness())
    self.UpdateStrandEccGui(options.GetStrandEcc())
    self.strand_profile_spinbox_.setValue(options.GetStrandProfileType())
    self.strand_mode_spinbox_.setValue(options.GetStrandMode())
    
  def UpdatePolyMode(self, value):
    self.GetOptions().SetPolyMode(value)
    self.ApplyOptions()
    
  def UpdateSplineDetail(self, value):
    self.GetOptions().SetSplineDetail(value)
    self.ApplyOptions()
    
  def UpdateArcDetail(self, value):
    self.GetOptions().SetArcDetail(value)
    self.ApplyOptions()
    
  def UpdateTubeRadius(self, value):
    self.GetOptions().SetTubeRadius(value)
    self.ApplyOptions()
    
  def UpdateSliderTubeRadius(self, value):
    self.GetOptions().SetTubeRadius(value/10.0)
    self.ApplyOptions()
    
  def UpdateTubeRatio(self, value):
    self.GetOptions().SetTubeRatio(value)
    self.ApplyOptions()
    
  def UpdateSliderTubeRatio(self, value):
    self.GetOptions().SetTubeRatio(value/10.0)
    self.ApplyOptions()
    
  def UpdateTubeProfileType(self, value):
    self.GetOptions().SetTubeProfileType(value)
    self.ApplyOptions()
    
  def UpdateHelixWidth(self, value):
    self.GetOptions().SetHelixWidth(value) 
    self.ApplyOptions()
    
  def UpdateSliderHelixWidth(self, value):
    self.GetOptions().SetHelixWidth(value/10.0)
    self.ApplyOptions()
    
  def UpdateHelixThickness(self, value):
    self.GetOptions().SetHelixThickness(value)
    self.ApplyOptions()
    
  def UpdateSliderHelixThickness(self, value):
    self.GetOptions().SetHelixThickness(value/10.0)
    self.ApplyOptions()
    
  def UpdateHelixEcc(self, value):
    self.GetOptions().SetHelixEcc(value)
    self.ApplyOptions()
    
  def UpdateHelixProfileType(self, value):
    self.GetOptions().SetHelixProfileType(value)
    self.ApplyOptions()

  def UpdateHelixMode(self, value):
    self.GetOptions().SetHelixMode(value)
    self.ApplyOptions()
    
  def UpdateSliderHelixEcc(self, value):
    self.GetOptions().SetHelixEcc(value/10.0)
    self.ApplyOptions()
    
  def UpdateStrandWidth(self, value):
    self.GetOptions().SetStrandWidth(value)
    self.ApplyOptions()
    
  def UpdateSliderStrandWidth(self, value):
    self.GetOptions().SetStrandWidth(value/10.0)
    self.ApplyOptions()
    
  def UpdateStrandThickness(self, value):
    self.GetOptions().SetStrandThickness(value)
    self.ApplyOptions()
    
  def UpdateSliderStrandThickness(self, value):
    self.GetOptions().SetStrandThickness(value/10.0)
    self.ApplyOptions()
    
  def UpdateStrandEcc(self, value):
    self.GetOptions().SetStrandEcc(value)
    self.ApplyOptions()
    
  def UpdateSliderStrandEcc(self, value):
    self.GetOptions().SetStrandEcc(value/10.0)
    self.ApplyOptions()
    
  def UpdateStrandProfileType(self, value):
    self.GetOptions().SetStrandProfileType(value)
    self.ApplyOptions()
    
  def UpdateStrandMode(self, value):
    self.GetOptions().SetStrandMode(value)
    self.ApplyOptions()
    
  def UpdateTubeRadiusGui(self,value):
    value = round(value, 2)
    if(abs(value*10.0 - self.width_tube_slider_.value())>=self.width_tube_slider_.singleStep()):
      self.width_tube_slider_.setValue(value*10.0)
    if(abs(value - self.width_tube_spinbox_.value())>=self.width_tube_spinbox_.singleStep()):
      self.width_tube_spinbox_.setValue(value)

  def UpdateTubeRatioGui(self,value):
    value = round(value, 2)
    if(abs(value*10.0 - self.thickness_tube_slider_.value())>=self.thickness_tube_slider_.singleStep()):
      self.thickness_tube_slider_.setValue(value*10.0)
    if(abs(value - self.thickness_tube_spinbox_.value())>=self.thickness_tube_spinbox_.singleStep()):
      self.thickness_tube_spinbox_.setValue(value)

  def UpdateHelixWidthGui(self, value):
    value = round(value, 2)
    if(abs(value*10.0 - self.width_helix_slider_.value())>=self.width_helix_slider_.singleStep()):
      self.width_helix_slider_.setValue(value*10.0)
    if(abs(value - self.width_helix_spinbox_.value())>=self.width_helix_spinbox_.singleStep()):
      self.width_helix_spinbox_.setValue(value)
    
  def UpdateHelixThicknessGui(self, value):
    value = round(value, 2)
    if(abs(value*10.0 - self.thickness_helix_slider_.value())>=self.thickness_helix_slider_.singleStep()):
      self.thickness_helix_slider_.setValue(value*10.0)
    if(abs(value - self.thickness_helix_spinbox_.value())>=self.thickness_helix_spinbox_.singleStep()):
      self.thickness_helix_spinbox_.setValue(value)
    
  def UpdateHelixEccGui(self, value):
    value = round(value, 2)
    if(abs(value*10.0 - self.ecc_helix_slider_.value())>=self.ecc_helix_slider_.singleStep()):
      self.ecc_helix_slider_.setValue(value*10.0)
    if(abs(value - self.ecc_helix_spinbox_.value())>=self.ecc_helix_spinbox_.singleStep()):
      self.ecc_helix_spinbox_.setValue(value)
    
  def UpdateStrandWidthGui(self, value):
    value = round(value, 2)
    if(abs(value*10.0 - self.width_strand_slider_.value())>=self.width_strand_slider_.singleStep()):
      self.width_strand_slider_.setValue(value*10.0)
    if(abs(value - self.width_strand_spinbox_.value())>=self.width_strand_spinbox_.singleStep()):
      self.width_strand_spinbox_.setValue(value)

  def UpdateStrandThicknessGui(self, value):
    value = round(value, 2)
    if(abs(value*10.0 - self.thickness_strand_slider_.value())>=self.thickness_strand_slider_.singleStep()):
      self.thickness_strand_slider_.setValue(value*10.0)
    if(abs(value - self.thickness_strand_spinbox_.value())>=self.thickness_strand_spinbox_.singleStep()):
      self.thickness_strand_spinbox_.setValue(value)
    
  def UpdateStrandEccGui(self, value):
    value = round(value, 2)
    if(abs(value*10.0 - self.ecc_strand_slider_.value())>=self.ecc_strand_slider_.singleStep()):
      self.ecc_strand_slider_.setValue(value*10.0)
    if(abs(value - self.ecc_strand_spinbox_.value())>=self.ecc_strand_spinbox_.singleStep()):
      self.ecc_strand_spinbox_.setValue(value)

  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

from ost import gui
from ost import info
import ost
import re
from PyQt5 import QtCore, QtGui, QtNetwork
from ost.gui import FileLoader

"""
class BaseRemoteLoader(gui.RemoteSiteLoader):
  def __init__(self):
    gui.RemoteSiteLoader.__init__(self)
    self.networkmanager_=QtNetwork.QNetworkAccessManager()
    self.downloads_=dict()
    QtCore.QObject.connect(self.networkmanager_, QtCore.SIGNAL("finished(QNetworkReply*)"), self.DownloadFinished)

  def LoadById(self, id, selection=""):
    self.ById(id, selection)
  
  def ById(self, ids, selection=""):
    for id in re.findall('\w+', ids):
      file_name=self.GetFileName(id)
      file = QtCore.QFile(file_name)
      if(file.size()==0):
          url = QtCore.QUrl(self.GetUrl(id))
          request = QtNetwork.QNetworkRequest(url)
          reply = self.networkmanager_.get(request)
          self.downloads_[reply]=[id,selection]
      else:
        gui.FileLoader.LoadObject(str(file_name),str(selection))
    return None
    
  def IsImg(self):
    return False
    
  def IsDefault(self):
    return False

  def GetRemoteSiteName(self):
    pass  
  
  def GetUrl(self,id):
    pass
      
  def HandleError(self,message):
    pass
    
  def DownloadFinished(self, reply):
    file_name=self.GetFileName(self.downloads_[reply][0])
    file = QtCore.QFile(file_name)
    if(reply.error()!=QtNetwork.QNetworkReply.NoError or reply.bytesAvailable()==0):
      self.HandleError(reply.errorString());
      file.remove()
    else:
      if(self.downloads_.has_key(reply)):
        if(file.open(QtCore.QIODevice.WriteOnly)):
           file.write(reply.readAll());
        file.close();
        selection=self.downloads_[reply][1]
        del(self.downloads_[reply])
        gui.FileLoader.LoadObject(str(file_name),str(selection))

class GenericLoader(BaseRemoteLoader):
  EXT_NAME_ATTRIBUTE_NAME = "ExtName"
  URL_ATTRIBUTE_NAME = "Url"
  UP_CASE_ATTRIBUTE_NAME = "UpCase"
  FILE_TYPE_ATTRIBUTE_NAME = "FileType"
  TMP_URL_ATTRIBUTE_NAME= "TmpUrl"
  IMG_ATTRIBUTE_NAME= "Img"
  DEFAULT_ATTRIBUTE_NAME= "Default"
  def __init__(self, name, url, up_case, file_type, tmp_url=None, img=False, default=False):
    BaseRemoteLoader.__init__(self)
    self.name_ = QtCore.QString(name)
    self.url_ = QtCore.QString(url)
    self.up_case_ = up_case
    self.img_ = img
    self.default_ = default
    if file_type is not None:
      self.file_type_ = QtCore.QString(file_type)
    else:
      self.file_type_ = None
    
    if tmp_url is not None:
      self.tmp_url_ = QtCore.QString(tmp_url)
    else:
      self.tmp_url_ = tmp_url
    
  def IsImg(self):
    return self.img_    

  def IsDefault(self):
    return self.default_
  
  def GetRemoteSiteName(self):
    return self.name_
  
  def GetUrl(self, id):
    formatted_id = id
    if self.up_case_:
      formatted_id = QtCore.QString(id).toUpper()
    url = QtCore.QString(self.url_)
    url.replace("${ID}",formatted_id)
    return str(url)
    
  def GetFileName(self,id):
    formatted_id = id
    if self.up_case_:
      formatted_id = QtCore.QString(id).toUpper()
    
    url = None
    if self.tmp_url_ is not None:
      url = self.tmp_url_ + QtCore.QDir.separator()
    else:
      url = QtCore.QDir.tempPath() + QtCore.QDir.separator()
    
    file_type = None 
    if self.file_type_ is None:
      remote_url = QtCore.QString(self.GetUrl(id))
      index = remote_url.lastIndexOf(".")
      if(index >=0):
        file_type = remote_url.right(remote_url.size()-index-1)
    if file_type is None:
      file_type = self.file_type_
    if len(self.name_)>0:
      return url+formatted_id +"_"+self.name_+"."+file_type
    else:
      return url+formatted_id+"."+file_type
  
  def HandleError(self, message):
    ost.LogError(str("Could not download file\n"+message+""))
   
  def ToInfo(self,group):
    group.SetAttribute(GenericLoader.EXT_NAME_ATTRIBUTE_NAME, str(self.name_))
    group.SetAttribute(GenericLoader.URL_ATTRIBUTE_NAME, str(self.url_))
    if self.up_case_:
      group.SetAttribute(GenericLoader.UP_CASE_ATTRIBUTE_NAME, str(self.up_case_))
    if self.file_type_ is not None:
      group.SetAttribute(GenericLoader.FILE_TYPE_ATTRIBUTE_NAME, str(self.file_type_))
    if self.tmp_url_ is not None:
      group.SetAttribute(GenericLoader.TMP_URL_ATTRIBUTE_NAME, str(self.tmp_url_))
    if self.img_ is not None:
      group.SetAttribute(GenericLoader.IMG_ATTRIBUTE_NAME, str(int(self.img_)))
    if self.default_ is not None:
      group.SetAttribute(GenericLoader.DEFAULT_ATTRIBUTE_NAME, str(int(self.default_)))
      
  @staticmethod
  def FromInfo(group):
    name = None
    url = None
    up_case = False
    file_type = None
    tmp_url = None
    img = False
    default = False
    
    if group.HasAttribute(GenericLoader.EXT_NAME_ATTRIBUTE_NAME):
      name = QtCore.QString(group.GetAttribute(GenericLoader.EXT_NAME_ATTRIBUTE_NAME))
    
    if group.HasAttribute(GenericLoader.URL_ATTRIBUTE_NAME):
      url = QtCore.QString(group.GetAttribute(GenericLoader.URL_ATTRIBUTE_NAME))
      
    if group.HasAttribute(GenericLoader.UP_CASE_ATTRIBUTE_NAME):
      up_case = bool(group.GetAttribute(GenericLoader.UP_CASE_ATTRIBUTE_NAME))  
    
    if group.HasAttribute(GenericLoader.FILE_TYPE_ATTRIBUTE_NAME):
      file_type = QtCore.QString(group.GetAttribute(GenericLoader.FILE_TYPE_ATTRIBUTE_NAME))
      
    if group.HasAttribute(GenericLoader.TMP_URL_ATTRIBUTE_NAME):
      tmp_url = QtCore.QString(group.GetAttribute(GenericLoader.TMP_URL_ATTRIBUTE_NAME))
    
    if group.HasAttribute(GenericLoader.IMG_ATTRIBUTE_NAME):
      img = bool(int(group.GetAttribute(GenericLoader.IMG_ATTRIBUTE_NAME)))
      
    if group.HasAttribute(GenericLoader.DEFAULT_ATTRIBUTE_NAME):
      default = bool(int(group.GetAttribute(GenericLoader.DEFAULT_ATTRIBUTE_NAME)))

    return GenericLoader(name, url, up_case, file_type, tmp_url, img, default)
"""



import unittest
from ost import conop, mol


class TestProcessor(unittest.TestCase):
  def testPyWrap(self):
    class MyProc(conop.Processor):
      def __init__(self):
        conop.Processor.__init__(self)
        self.count =0
      def DoProcess(self, diag, ent):
        self.count+=1
    p = MyProc()
    ent = mol.CreateEntity()
    p.Process(ent)
    self.assertEqual(p.count, 1)

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()


